#include "fintrf.h"
!     
!=======================================================================
! Gateway subroutine
subroutine mexfunction(nlhs, plhs, nrhs, prhs)

! Declarations
implicit none

! mexFunction arguments:
mwPointer plhs(*), prhs(*)
integer nlhs, nrhs

! Function declarations:
mwPointer mxGetPr
mwPointer mxCreateDoubleMatrix, mxCreateNumericArray
mwPointer mxGetM, mxGetN

! Pointers to input/output mxArrays:
mwPointer pr1 ,pr2 ,pr3 ,pr4 ,pr5 ,pr6 ,pr7 ,pr8 ,pr9 ,pr10,&
          pr11,pr12,pr13,&
          pr1_out,pr2_out,pr3_out,pr4_out

! Array information:
integer:: nx,ny,Nmax,firstOn,order_app
integer convertToInt

! Arguments for mxCreateNumericArray
integer*4 mxClassIDFromClassName
integer*4 classid
integer*4 complexflag
mwSize ndim3,ndim4,ndim5,ndim6
mwSize dims3(3),dims4(4),dims5(5),dims6(6)

!-----------------------------------------------------------------------
! Check for proper number of arguments. 
if(nrhs .ne. 13) then
   call mexErrMsgIdAndTxt ('MATLAB:dblmat:nInput','13 input required.')
end if
if(nlhs .gt. 4) then
   call mexErrMsgIdAndTxt ('MATLAB:dblmat:nOutput','Too many output arguments.')
end if
         
! Create Fortran arrays from the input argument.
pr1  = mxGetPr(prhs(1))
pr2  = mxGetPr(prhs(2))
pr3  = mxGetPr(prhs(3))
pr4  = mxGetPr(prhs(4))
pr5  = mxGetPr(prhs(5))
pr6  = mxGetPr(prhs(6))
pr7  = mxGetPr(prhs(7))
pr8  = mxGetPr(prhs(8))
pr9  = mxGetPr(prhs(9))
pr10 = mxGetPr(prhs(10))
pr11 = mxGetPr(prhs(11))
pr12 = mxGetPr(prhs(12))
pr13 = mxGetPr(prhs(13))

! Get the size of the input array.
!call mxCopyPtrToInteger4(pr9, Nmax, 1)
!call mxCopyPtrToInteger4(pr10, firstOn, 1)
!call mxCopyPtrToInteger4(pr12, ny, 1)
!call mxCopyPtrToInteger4(pr13, nx, 1)
Nmax      = convertToInt(%VAL(pr9))
firstOn   = convertToInt(%VAL(pr10))
order_app = convertToInt(%VAL(pr11))
ny        = convertToInt(%VAL(pr12))
nx        = convertToInt(%VAL(pr13))


! Create matrix for the return argument.
!for Ey: ny * (firstOn+Nmax) * nx
classid = mxClassIDFromClassName('double')        
complexflag = 0                                 ! 0 for real data
ndim3    = 3
dims3(1) = ny
dims3(2) = Nmax+firstOn
dims3(3) = nx
plhs(1) = mxCreateNumericArray(ndim3, dims3, classid, complexflag)
pr1_out = mxGetPr(plhs(1))

!for Eyy: ny * (firstOn+Nmax) * nx * nx
ndim4    = 4                                     
dims4(1) = ny
dims4(2) = Nmax+firstOn
dims4(3) = nx
dims4(4) = nx
plhs(2) = mxCreateNumericArray(ndim4, dims4, classid, complexflag)
pr2_out = mxGetPr(plhs(2))

!for Eyyy: ny * (firstOn+Nmax) * nx * nx * nx
ndim5    = 5
dims5(1) = ny
dims5(2) = Nmax+firstOn
dims5(3) = nx
dims5(4) = nx
dims5(5) = nx
plhs(3) = mxCreateNumericArray(ndim5, dims5, classid, complexflag)
pr3_out = mxGetPr(plhs(3))

!for E4y: ny * (firstOn+Nmax) * nx * nx * nx * nx
ndim6    = 6
dims6(1) = ny
dims6(2) = Nmax+firstOn
dims6(3) = nx
dims6(4) = nx
dims6(5) = nx
dims6(6) = nx
plhs(4) = mxCreateNumericArray(ndim6, dims6, classid, complexflag)
pr4_out = mxGetPr(plhs(4))

! Call the computational routine.
!call fortranSub(%VAL(pr1_out),%VAL(pr2_out),%VAL(pr3_out),%VAL(pr4_out),&
!   %VAL(pr1) ,%VAL(pr2) ,%VAL(pr3) ,%VAL(pr4) ,%VAL(pr5) ,&
!   %VAL(pr6) ,%VAL(pr7) ,%VAL(pr8) ,%VAL(pr9) ,%VAL(pr10),&
!   %VAL(pr11),%VAL(pr12),%VAL(pr13))

call fortranSub(%VAL(pr1_out),%VAL(pr2_out),%VAL(pr3_out),%VAL(pr4_out),&
   %VAL(pr1) ,%VAL(pr2) ,%VAL(pr3) ,%VAL(pr4) ,%VAL(pr5) ,&
   %VAL(pr6) ,%VAL(pr7) ,%VAL(pr8) ,Nmax      ,firstOn   ,&
   order_app,ny,nx)
return
end

function convertToInt(matlab_dble) result( int_val )

    implicit none

    REAL*8, INTENT(IN):: matlab_dble    
    INTEGER*8:: int_val

    int_val = INT(matlab_dble)

end function convertToInt


!===============================================================
! This function computes the conditional moments up to k_period into the
! future for the variables in the g-function.
! In terms of the notation we solve: p(x,sig) = E_t[r(x_t+1,sig)]
! The computations are derived by using the Perturbation On Perturbation (POP) method.
! IMPORTANT: 1) This is for a "level" approximation under perfect foresight
!            2) The first time period in px, pxx, pxxx is the current time
!            period, i.e. we reproduce gx, gxx, gxxx, if firstOn = 1.
!
! Computational subroutine
subroutine fortranSub(px,pxx,pxxx,p4x,&
 gx,gxx,gxxx,g4x,hx,hxx,hxxx,h4x,k_period,firstOn,order_app,ny,nx)

IMPLICIT NONE
INTEGER*8, INTENT(IN) :: k_period,firstOn,order_app,ny,nx
REAL*8,  INTENT(OUT):: px(ny,k_period+firstOn,nx),pxx(ny,k_period+firstOn,nx,nx),&
                       pxxx(ny,k_period+firstOn,nx,nx,nx),p4x(ny,k_period+firstOn,nx,nx,nx,nx)
REAL*8,  INTENT(IN) :: gx(ny,nx),gxx(ny,nx,nx),gxxx(ny,nx,nx,nx),g4x(ny,nx,nx,nx,nx),&
                       hx(nx,nx),hxx(nx,nx,nx),hxxx(nx,nx,nx,nx),h4x(nx,nx,nx,nx,nx)

! Declaring the remaining variables
INTEGER*8 i,j,k,alfa1,alfa2,alfa3,alfa4,gama1,gama2,gama3,gama4,idx,startIndex
REAL*8  rx(ny,nx),rxx(ny,nx**2),rxxx(ny,nx**3),r4x(ny,nx**4),tmp(ny,1),&
        kron_nx2_1(nx**2,1),kron_nx2_2(nx**2,1),kron_nx2_3(nx**2,1),&
        kron_nx2_4(nx**2,1),kron_nx2_5(nx**2,1),&
        kron_nx3_1(nx**3,1),kron_nx3_2(nx**3,1),kron_nx3_3(nx**3,1),&
        kron_nx3_4(nx**3,1),kron_nx3_5(nx**3,1),&
        kron_nx4_1(nx**4,1),&
        tmp1(ny,1),tmp2_5(ny,1),tmp6_7(ny,1),tmp8(ny,1),tmp9_10(ny,1),tmp11(ny,1),&
        tmp12_14(ny,1)

character*120 line


! To print to a file
open(unit=6,file='tmp.txt',status='unknown')
!write(6,*) 'nx = ',nx,'ny = ',ny
!write(6,*) 'k_period =',k_period
!write(6,*) 'firstOn =',firstOn
!write(6,*) 'order_app=',order_app
!write(6,*) 'size(g4x,1)=',size(g4x,1)
!write(6,*) 'size(g4x,2)=',size(g4x,2)
!write(6,*) 'size(g4x,3)=',size(g4x,3)
!write(6,*) 'size(g4x,4)=',size(g4x,4)
!write(6,*) 'size(g4x,5)=',size(g4x,5)
!write(6,*) 'size(hx,1)=',size(hx,1)
!write(6,*) 'size(hx,2)=',size(hx,2)
!write(6,*) 'size(hxx,1)=',size(hxx,1)
!write(6,*) 'size(hxx,2)=',size(hxx,2)
!write(6,*) 'size(hxx,3)=',size(hxx,3)
!write(6,*) 'size(hxxx,1)=',size(hxxx,1)
!write(6,*) 'size(hxxx,2)=',size(hxxx,2)
!write(6,*) 'size(hxxx,3)=',size(hxxx,3)
!write(6,*) 'size(hxxx,4)=',size(hxxx,4)


! The first time period
IF (firstOn == 1) THEN
   px(1:ny,1,1:nx)                  = gx
   pxx(1:ny,1,1:nx,1:nx)            = gxx
   pxxx(1:ny,1,1:nx,1:nx,1:nx)      = gxxx
   p4x(1:ny,1,1:nx,1:nx,1:nx,1:nx)  = g4x
   startIndex                       = 2
ELSE
   startIndex     = 1
END IF

rx   = gx
rxx  = RESHAPE(gxx ,(/ny,nx**2/))
rxxx = RESHAPE(gxxx,(/ny,nx**3/))
r4x  = RESHAPE(g4x ,(/ny,nx**4/))
    
DO j=startIndex,k_period+firstOn
   ! ************** The first order effects *****************
   px(:,j,:) = MATMUL(rx,hx)
      
   !*************** The second order effects ***************
   IF (order_app > 1) THEN
      DO alfa1=1,nx
         DO alfa2=alfa1,nx
            DO k=1,nx
               kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa1)*hx(:,alfa2)
            END DO
            pxx(:,j,alfa1:alfa1,alfa2) = MATMUL(rxx,kron_nx2_1)&
                                       + MATMUL(rx,hxx(1:nx,alfa1:alfa1,alfa2))
         END DO
         IF (alfa1 > 1) THEN
            pxx(:,j,alfa1,1:alfa1-1) = pxx(:,j,1:alfa1-1,alfa1)
         END IF
      END DO
   END IF
   ! ***************** The third order effects *****************
   IF (order_app > 2) THEN
      DO alfa1=1,nx
         DO alfa2=alfa1,nx
            DO alfa3=alfa2,nx
               !tmp = rxxx*kron(hx(:,alfa1),kron(hx(:,alfa2),hx(:,alfa3)))
               DO k=1,nx
                 kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa2)*hx(:,alfa3)
               END DO
               DO k=1,nx
                  kron_nx3_1(nx**2*(k-1)+1:nx**2*k,1) = hx(k,alfa1)*kron_nx2_1(:,1)
               END DO
               tmp = MATMUL(rxxx,kron_nx3_1)

               !kron(hx(:,alfa1:alfa1),hxx(:,alfa2:alfa2,alfa3)
               DO k=1,nx
                 kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa1)*hxx(:,alfa2,alfa3)
               END DO
               !kron(hxx(:,alfa1:alfa1,alfa3),hx(:,alfa2:alfa2))
               DO k=1,nx
                 kron_nx2_2(nx*(k-1)+1:nx*k,1) = hxx(k,alfa1,alfa3)*hx(:,alfa2)
               END DO
               !kron(hxx(:,alfa1:alfa1,alfa2),hx(:,alfa3:alfa3))
               DO k=1,nx
                 kron_nx2_3(nx*(k-1)+1:nx*k,1) = hxx(k,alfa1,alfa2)*hx(:,alfa3)
               END DO

               pxxx(:,j:j,alfa1,alfa2,alfa3) = tmp &
                             +MATMUL(rxx,kron_nx2_1+kron_nx2_2+kron_nx2_3)&
                             +MATMUL(rx,hxxx(:,alfa1:alfa1,alfa2,alfa3))
                                   
               ! Using symmetry for alfa1 and alfa2
               IF (alfa1 == alfa2 .AND. alfa2 .NE. alfa3) THEN  !alfa1==alfa2~=alfa3
                  !pxxx(:,j,alfa1,alfa1,alfa3)= pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa1,alfa3,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa3,alfa1,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3)
               END IF
               ! Using symmetry for alfa2 and alfa3
               IF (alfa1 .NE. alfa2 .AND. alfa2 == alfa3) THEN   !alfa1~=alfa2==alfa3
                  !pxxx(:,j,alfa1,alfa2,alfa2)= pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa2,alfa1,alfa2) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa2,alfa2,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3)
               END IF
               ! Using symmetry for alfa1,alfa2, and alfa3
               IF (alfa1 .NE. alfa2 .AND. alfa1 .NE. alfa3 .AND. alfa2 .NE. alfa3) THEN !alfa1~=alfa2~=alfa3
                  !pxxx(:,j,alfa1,alfa2,alfa3) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa1,alfa3,alfa2) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa3,alfa1,alfa2) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa3,alfa2,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa2,alfa3,alfa1) = pxxx(:,j,alfa1,alfa2,alfa3)
                  pxxx(:,j,alfa2,alfa1,alfa3) = pxxx(:,j,alfa1,alfa2,alfa3)
               END IF
            END DO
         END DO
      END DO
   END IF
   ! ***************** The fourth order effects *****************      
   IF (order_app > 3) THEN
      DO alfa1=1,nx
         DO alfa2=alfa1,nx
            DO alfa3=alfa2,nx
               DO alfa4=alfa3,nx
                  !tmp1 = r4x*kron(hx(:,alfa1),kron(hx(:,alfa2),kron(hx(:,alfa3),hx(:,alfa4))));
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa3)*hx(:,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx2_2(nx*(k-1)+1:nx*k,1) = hx(k,alfa1)*hx(:,alfa2)
                  END DO
                  DO k=1,nx**2
                     kron_nx4_1(nx**2*(k-1)+1:nx**2*k,1) = kron_nx2_2(k,1)*kron_nx2_1(:,1)
                  END DO
                  tmp1 = MATMUL(r4x,kron_nx4_1)
                  
                  !rxxx*(kron(hx(:,alfa1),kron(hx(:,alfa2),squeeze(hxx(:,alfa3,alfa4)))) + ...  %2)
                  !                kron(hx(:,alfa1),kron(squeeze(hxx(:,alfa2,alfa4)),hx(:,alfa3))) + ...  %3)
                  !                kron(squeeze(hxx(:,alfa1,alfa4)),kron(hx(:,alfa2),hx(:,alfa3))) + ...  %4)
                  !                kron(hx(:,alfa1),kron(squeeze(hxx(:,alfa2,alfa3)),hx(:,alfa4))) )+ ... %5)
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa2)*hxx(:,alfa3,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx3_1(nx**2*(k-1)+1:nx**2*k,1) = hx(k,alfa1)*kron_nx2_1(:,1)
                  END DO
                  DO k=1,nx
                     kron_nx2_2(nx*(k-1)+1:nx*k,1) = hxx(k,alfa2,alfa4)*hx(:,alfa3)
                  END DO
                  DO k=1,nx
                     kron_nx3_2(nx**2*(k-1)+1:nx**2*k,1) = hx(k,alfa1)*kron_nx2_2(:,1)
                  END DO
                  DO k=1,nx
                     kron_nx2_3(nx*(k-1)+1:nx*k,1) = hx(k,alfa2)*hx(:,alfa3)
                  END DO
                  DO k=1,nx
                     kron_nx3_3(nx**2*(k-1)+1:nx**2*k,1) = hxx(k,alfa1,alfa4)*kron_nx2_3(:,1)
                  END DO
                  DO k=1,nx
                     kron_nx2_4(nx*(k-1)+1:nx*k,1) = hxx(k,alfa2,alfa3)*hx(:,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx3_4(nx**2*(k-1)+1:nx**2*k,1) = hx(k,alfa1)*kron_nx2_4(:,1)
                  END DO
                  tmp2_5 = MATMUL(rxxx,kron_nx3_1+kron_nx3_2+kron_nx3_3+kron_nx3_4)
                  
                  !          rxx*(kron(hx(:,alfa1),squeeze(hxxx(:,alfa2,alfa3,alfa4))) +...               %6)
                  !               kron(squeeze(hxx(:,alfa1,alfa4)),squeeze(hxx(:,alfa2,alfa3))) )+...     %7)
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa1)*hxxx(:,alfa2,alfa3,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx2_2(nx*(k-1)+1:nx*k,1) = hxx(k,alfa1,alfa4)*hxx(:,alfa2,alfa3)
                  END DO
                  tmp6_7 = MATMUL(rxx,kron_nx2_1+kron_nx2_2)

                  !rxxx*kron(squeeze(hxx(:,alfa1,alfa3)),kron(hx(:,alfa2),hx(:,alfa4)))  + ...  %8)
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa2)*hx(:,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx3_1(nx**2*(k-1)+1:nx**2*k,1) = hxx(k,alfa1,alfa3)*kron_nx2_1(:,1)
                  END DO
                  tmp8 = MATMUL(rxxx,kron_nx3_1)

                  !rxx*(kron(squeeze(hxx(:,alfa1,alfa3)),squeeze(hxx(:,alfa2,alfa4))) + ...     %9)
                  !               kron(squeeze(hxxx(:,alfa1,alfa3,alfa4)),hx(:,alfa2))) +...    %10)
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hxx(k,alfa1,alfa3)*hxx(:,alfa2,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx2_2(nx*(k-1)+1:nx*k,1) = hxxx(k,alfa1,alfa3,alfa4)*hx(:,alfa2)
                  END DO
                  tmp9_10 = MATMUL(rxx,kron_nx2_1 + kron_nx2_2)

                  !rxxx*kron(squeeze(hxx(:,alfa1,alfa2)),kron(hx(:,alfa3),hx(:,alfa4)))  + ...  %11)
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hx(k,alfa3)*hx(:,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx3_1(nx**2*(k-1)+1:nx**2*k,1) = hxx(k,alfa1,alfa2)*kron_nx2_1(:,1)
                  END DO
                  tmp11 = MATMUL(rxxx,kron_nx3_1)                  

                  !rxx*(kron(squeeze(hxx(:,alfa1,alfa2)),squeeze(hxx(:,alfa3,alfa4))) + ...     %12)
                  !               kron(squeeze(hxxx(:,alfa1,alfa2,alfa4)),hx(:,alfa3)) +...     %13)
                  !               kron(squeeze(hxxx(:,alfa1,alfa2,alfa3)),hx(:,alfa4)))+...     %14)
                  DO k=1,nx
                     kron_nx2_1(nx*(k-1)+1:nx*k,1) = hxx(k,alfa1,alfa2)*hxx(:,alfa3,alfa4)
                  END DO
                  DO k=1,nx
                     kron_nx2_2(nx*(k-1)+1:nx*k,1) = hxxx(k,alfa1,alfa2,alfa4)*hx(:,alfa3)
                  END DO
                  DO k=1,nx
                     kron_nx2_3(nx*(k-1)+1:nx*k,1) = hxxx(k,alfa1,alfa2,alfa3)*hx(:,alfa4)
                  END DO
                  tmp12_14 = MATMUL(rxx,kron_nx2_1 + kron_nx2_2 + kron_nx2_3)

                  p4x(:,j,alfa1:alfa1,alfa2,alfa3,alfa4) =  tmp1 + tmp2_5 + tmp6_7 + tmp8 + tmp9_10 + &
                                                      tmp11 + tmp12_14 + &
                                                      MATMUL(rx,h4x(:,alfa1:alfa1,alfa2,alfa3,alfa4))
                  
                  !1) Three identical and one different, for instance(1,1,1,2)
                  IF (alfa1 == alfa2 .AND. alfa1 == alfa3 .AND. alfa1.NE.alfa4) THEN
                     !p4x(:,j,alfa1,alfa2,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa1,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa4,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa1,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
                  !2) Three identical and one different, for instance(1,2,2,2)
                  IF (alfa1.NE.alfa2 .AND. alfa2 == alfa3 .AND. alfa3 == alfa4) THEN
                     !p4x(:,j,alfa1,alfa2,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa1,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa2,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa2,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
                  !3) Two identical and two different, for instance(1,1,2,2)
                  IF (alfa1 == alfa2 .AND. alfa3 == alfa4 .AND. alfa1 .NE. alfa3) THEN
                     !p4x(:,j,alfa1,alfa1,alfa3,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa3,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa1,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa3,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa1,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa3,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
                  !4) To identical but three different, for instance (1,2,3,3)
                  IF (alfa1 .NE. alfa2 .AND. alfa3 == alfa4 .AND. alfa1.NE. alfa3 .AND. alfa2.NE.alfa3) THEN
                     !p4x(:,j,alfa1,alfa2,alfa3,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa1,alfa3,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa3,alfa3,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa3,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa1,alfa3,alfa3,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa3,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa3,alfa2,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa1,alfa3,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa3,alfa2,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa1,alfa2,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa1,alfa3,alfa2,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa3,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
                  !5) Two identical but three different, for instance (1,1,2,3)
                  IF (alfa1 == alfa2 .AND. alfa3.NE.alfa4 .AND. alfa1.NE.alfa3 .AND. alfa2 .NE. alfa3) THEN
                     !p4x(:,j,alfa1,alfa1,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa1,alfa4,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa3,alfa4,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa3,alfa1,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                           
                     p4x(:,j,alfa3,alfa1,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa1,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa1,alfa3,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa4,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa1,alfa3,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa4,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa3,alfa1,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa1,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
                  !6) Two identical but three different, for instance (1,2,2,3)
                  IF (alfa2 == alfa3 .AND. alfa1 .NE. alfa4 .AND. alfa1 .NE. alfa2 .AND. alfa3 .NE. alfa4) THEN
                     !p4x(:,j,alfa1,alfa2,alfa2,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa2,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa2,alfa4,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa1,alfa2,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa2,alfa4,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa1,alfa4,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa4,alfa2,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa2,alfa4,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa4,alfa1,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa4,alfa2,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                            
                     p4x(:,j,alfa2,alfa2,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa2,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
                  !7) all different
                  IF (alfa1.NE.alfa2 .AND. alfa1.NE.alfa3 .AND. alfa1.NE.alfa4 .AND. alfa2.NE.alfa3 &
                     .AND. alfa2.NE.alfa4 .AND. alfa3.NE.alfa4) THEN
                     p4x(:,j,alfa4,alfa3,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa3,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa2,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa2,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa1,alfa3,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa4,alfa1,alfa2,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa4,alfa2,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa4,alfa1,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa2,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa2,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa1,alfa4,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa3,alfa1,alfa2,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa4,alfa3,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa4,alfa1,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa3,alfa4,alfa1) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa3,alfa1,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa1,alfa4,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa2,alfa1,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa4,alfa3,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa4,alfa2,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa3,alfa4,alfa2) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa3,alfa2,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     p4x(:,j,alfa1,alfa2,alfa4,alfa3) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                     !p4x(:,j,alfa1,alfa2,alfa3,alfa4) = p4x(:,j,alfa1,alfa2,alfa3,alfa4)
                  END IF
               END DO
            END DO
         END DO
      END DO 
   END IF 
 
   ! Updating rx
   rx      = px(:,j,1:nx)
   rxx     = RESHAPE(pxx(:,j,1:nx,1:nx),(/ny,nx**2/))
   rxxx    = RESHAPE(pxxx(:,j,1:nx,1:nx,1:nx),(/ny,nx**3/))
   r4x     = RESHAPE(p4x(:,j,1:nx,1:nx,1:nx,1:nx),(/ny,nx**4/))
END DO


END SUBROUTINE fortranSub

