function [ResEuler,yGrid] = EulerEqErrorPruning(model,orderApp,setup)
 
% Some indices 
ny      = model.ny;        %Number of output variables
nx      = model.nx;        %Number of state variables 
derivs  = model.derivs;    %Deviatives as stored by Levintal 
 
%Unfold params
DELTA= model.params.DELTA;
BETTA= model.params.BETTA;
GAMA= model.params.GAMA;
PHI= model.params.PHI;
ALFA= model.params.ALFA;
RHOA= model.params.RHOA;
 
% The raw grid for the states 
xGrid_f = setup.xSim_decom.xt_f; 
if orderApp >= 2 
    xGrid_s = setup.xSim_decom.xt_s; 
end 
if orderApp >= 3 
    xGrid_rd = setup.xSim_decom.xt_rd; 
end 
if orderApp >= 4 
    xGrid_4th = setup.xSim_decom.xt_4th; 
end
if orderApp >= 5 
    xGrid_5th = setup.xSim_decom.xt_5th; 
end
 
 
% The grid for numerical itegration 
tmp_e        = cell(1,model.ne); 
tmp_w        = cell(1,model.ne); 
for i=1:model.ne 
    tmp_e(i) = mat2cell(setup.GH_e,length(setup.GH_e),1); 
    tmp_w(i) = mat2cell(setup.GH_w,length(setup.GH_w),1); 
end 
eGrid = cartprod(tmp_e); 
wGrid = prod(cartprod(tmp_w),2); 
 
%% Building the residuals 
f        = zeros(ny+nx,1); 
ResEuler = zeros(size(xGrid_f,2),ny+nx); 
yGrid    = zeros(size(xGrid_f,2),ny); 
% Setting the dimensions for the state components 
xf_cu    = [zeros(nx,1);1]; 
xs_cu    = [zeros(nx,1);0]; 
xrd_cu   = [zeros(nx,1);0]; 
x4th_cu  = [zeros(nx,1);0]; 
x5th_cu  = [zeros(nx,1);0]; 
 
xf_cup   = [zeros(nx,1);1]; 
xs_cup   = [zeros(nx,1);0]; 
xrd_cup  = [zeros(nx,1);0]; 
x4th_cup = [zeros(nx,1);0]; 
x5th_cup = [zeros(nx,1);0]; 
 
for k = 1:size(xGrid_f,2) 
    xf_cu    = xGrid_f(:,k); 
    if orderApp>=2 
        xs_cu  = xGrid_s(:,k); 
    end 
    if orderApp>=3 
        xrd_cu  = xGrid_rd(:,k); 
    end 
    if orderApp>=4 
        x4th_cu  = xGrid_4th(:,k); 
    end 
    if orderApp>=5 
        x5th_cu  = xGrid_5th(:,k); 
    end 
 
    % Getting the states at time t 
    k1_cu = model.h0(1,1) + xf_cu(1,1) + xs_cu(1,1) + xrd_cu(1,1) + x4th_cu(1,1) + x5th_cu(1,1);
    k2_cu = model.h0(2,1) + xf_cu(2,1) + xs_cu(2,1) + xrd_cu(2,1) + x4th_cu(2,1) + x5th_cu(2,1);
    k3_cu = model.h0(3,1) + xf_cu(3,1) + xs_cu(3,1) + xrd_cu(3,1) + x4th_cu(3,1) + x5th_cu(3,1);
    k4_cu = model.h0(4,1) + xf_cu(4,1) + xs_cu(4,1) + xrd_cu(4,1) + x4th_cu(4,1) + x5th_cu(4,1);
    k5_cu = model.h0(5,1) + xf_cu(5,1) + xs_cu(5,1) + xrd_cu(5,1) + x4th_cu(5,1) + x5th_cu(5,1);
    k6_cu = model.h0(6,1) + xf_cu(6,1) + xs_cu(6,1) + xrd_cu(6,1) + x4th_cu(6,1) + x5th_cu(6,1);
    k7_cu = model.h0(7,1) + xf_cu(7,1) + xs_cu(7,1) + xrd_cu(7,1) + x4th_cu(7,1) + x5th_cu(7,1);
    k8_cu = model.h0(8,1) + xf_cu(8,1) + xs_cu(8,1) + xrd_cu(8,1) + x4th_cu(8,1) + x5th_cu(8,1);
    k9_cu = model.h0(9,1) + xf_cu(9,1) + xs_cu(9,1) + xrd_cu(9,1) + x4th_cu(9,1) + x5th_cu(9,1);
    k10_cu = model.h0(10,1) + xf_cu(10,1) + xs_cu(10,1) + xrd_cu(10,1) + x4th_cu(10,1) + x5th_cu(10,1);
    a1_cu = model.h0(11,1) + xf_cu(11,1) + xs_cu(11,1) + xrd_cu(11,1) + x4th_cu(11,1) + x5th_cu(11,1);
    a2_cu = model.h0(12,1) + xf_cu(12,1) + xs_cu(12,1) + xrd_cu(12,1) + x4th_cu(12,1) + x5th_cu(12,1);
    a3_cu = model.h0(13,1) + xf_cu(13,1) + xs_cu(13,1) + xrd_cu(13,1) + x4th_cu(13,1) + x5th_cu(13,1);
    a4_cu = model.h0(14,1) + xf_cu(14,1) + xs_cu(14,1) + xrd_cu(14,1) + x4th_cu(14,1) + x5th_cu(14,1);
    a5_cu = model.h0(15,1) + xf_cu(15,1) + xs_cu(15,1) + xrd_cu(15,1) + x4th_cu(15,1) + x5th_cu(15,1);
    a6_cu = model.h0(16,1) + xf_cu(16,1) + xs_cu(16,1) + xrd_cu(16,1) + x4th_cu(16,1) + x5th_cu(16,1);
    a7_cu = model.h0(17,1) + xf_cu(17,1) + xs_cu(17,1) + xrd_cu(17,1) + x4th_cu(17,1) + x5th_cu(17,1);
    a8_cu = model.h0(18,1) + xf_cu(18,1) + xs_cu(18,1) + xrd_cu(18,1) + x4th_cu(18,1) + x5th_cu(18,1);
    a9_cu = model.h0(19,1) + xf_cu(19,1) + xs_cu(19,1) + xrd_cu(19,1) + x4th_cu(19,1) + x5th_cu(19,1);
    a10_cu = model.h0(20,1) + xf_cu(20,1) + xs_cu(20,1) + xrd_cu(20,1) + x4th_cu(20,1) + x5th_cu(20,1);
 
    % Getting the states at time t+1 
    xf_cup(1:end-1,1)     = derivs.hx*xf_cu; 
    if orderApp>=2 
        xf2_cu            = kron(xf_cu,xf_cu); 
        xs_cup(1:end-1,1) = derivs.hx*xs_cu + derivs.hxx*xf2_cu/2; 
    end 
    if orderApp>=3 
        xf3_cu             = kron(xf2_cu,xf_cu); 
        xf_xs_cu           = kron(xf_cu,xs_cu); 
        xrd_cup(1:end-1,1) = derivs.hx*xrd_cu + derivs.hxx*(2*xf_xs_cu)/2 ... 
                            + derivs.hxxx*xf3_cu/6; 
    end 
    if orderApp>=4 
        xf4_cu             = kron(xf3_cu,xf_cu); 
        xf2_xs_cu          = kron(xf_cu,xf_xs_cu); 
        xs2_cu             = kron(xs_cu,xs_cu); 
        xf_xrd_cu          = kron(xf_cu,xrd_cu); 
        x4th_cup(1:end-1,1)= derivs.hx*x4th_cu + derivs.hxx*(2*xf_xrd_cu + xs2_cu)/2 ... 
                            + derivs.hxxx*(3*xf2_xs_cu)/6 ... 
                            + derivs.hxxxx*xf4_cu/24; 
    end 
    if orderApp>=5 
        xf5_cu             = kron(xf4_cu,xf_cu); 
        xf3_xs_cu          = kron(xf_cu,xf2_xs_cu); 
        xf_xs2_cu          = kron(xf_cu,xs2_cu); 
        xf2_xrd_cu         = kron(xf_cu,xf_xrd_cu); 
        xs_xrd_cu          = kron(xs_cu,xrd_cu); 
        xf_x4th_cu         = kron(xf_cu,x4th_cu); 
        x5th_cup(1:end-1,1)= derivs.hx*x5th_cu ... 
                            + derivs.hxx*(2*xf_x4th_cu+2*xs_xrd_cu)/2 ... 
                            + derivs.hxxx*(3*xf2_xrd_cu+3*xf_xs2_cu)/6 ... 
                            + derivs.hxxxx*(4*xf3_xs_cu)/24 ... 
                            + derivs.hxxxxx*xf5_cu/120; 
    end 
    % Getting the controls at time t 
    if orderApp==1 
        yVec_cu = model.g0 + derivs.gx*xf_cu; 
    elseif orderApp==2 
        yVec_cu = model.g0 + derivs.gx*(xf_cu+xs_cu)+derivs.gxx*(xf2_cu)/2; 
    elseif orderApp==3 
        yVec_cu = model.g0 + derivs.gx*(xf_cu+xs_cu+xrd_cu)+derivs.gxx*(xf2_cu+2*xf_xs_cu)/2 ... 
            + derivs.gxxx*(xf3_cu)/6; 
    elseif orderApp==4 
        yVec_cu = model.g0 + derivs.gx*(xf_cu+xs_cu+xrd_cu+x4th_cu) ... 
            + derivs.gxx*(xf2_cu+2*xf_xs_cu+2*xf_xrd_cu+xs2_cu)/2 ... 
            + derivs.gxxx*(xf3_cu+3*xf2_xs_cu)/6 ... 
            + derivs.gxxxx*xf4_cu/24; 
    elseif orderApp==5 
        yVec_cu = model.g0 + derivs.gx*(xf_cu+xs_cu+xrd_cu+x4th_cu+x5th_cu) ... 
            + derivs.gxx*(xf2_cu+2*xf_xs_cu+2*xf_xrd_cu+2*xf_x4th_cu+xs2_cu+2*xs_xrd_cu)/2 ... 
            + derivs.gxxx*(xf3_cu+3*xf2_xs_cu+3*xf2_xrd_cu+3*xf_xs2_cu)/6 ... 
            + derivs.gxxxx*(xf4_cu+4*xf3_xs_cu)/24 ... 
            + derivs.gxxxxx*xf5_cu/120; 
    end 
    yGrid(k,:) = transpose((yVec_cu-model.g0)); 
    % The controls 
    c1_cu = yVec_cu(1,1);
    c2_cu = yVec_cu(2,1);
    c3_cu = yVec_cu(3,1);
    c4_cu = yVec_cu(4,1);
    c5_cu = yVec_cu(5,1);
    c6_cu = yVec_cu(6,1);
    c7_cu = yVec_cu(7,1);
    c8_cu = yVec_cu(8,1);
    c9_cu = yVec_cu(9,1);
    c10_cu = yVec_cu(10,1);
    iv1_cu = yVec_cu(11,1);
    iv2_cu = yVec_cu(12,1);
    iv3_cu = yVec_cu(13,1);
    iv4_cu = yVec_cu(14,1);
    iv5_cu = yVec_cu(15,1);
    iv6_cu = yVec_cu(16,1);
    iv7_cu = yVec_cu(17,1);
    iv8_cu = yVec_cu(18,1);
    iv9_cu = yVec_cu(19,1);
    iv10_cu = yVec_cu(20,1);
    la_cu = yVec_cu(21,1);
 
    Euler = zeros(ny+nx,1);
    xf_cup0 = xf_cup;
    for i=1:length(wGrid)
        % Adding shocks to the states 
        xf_cup(1:nx,1) = xf_cup0(1:nx,1) + model.eta*transpose(eGrid(i,:)); 
        % Getting controls at time t+1 
        if orderApp==1 
            yVec_cup  = model.g0 + derivs.gx*xf_cup; 
        elseif orderApp==2 
            xf2_cup   = kron(xf_cup,xf_cup); 
            yVec_cup  = model.g0 + derivs.gx*(xf_cup+xs_cup)+derivs.gxx*(xf2_cup)/2; 
        elseif orderApp==3 
            xf2_cup   = kron(xf_cup,xf_cup); 
            xf3_cup   = kron(xf2_cup,xf_cup); 
            xf_xs_cup = kron(xf_cup,xs_cup); 
            yVec_cup  = model.g0 + derivs.gx*(xf_cup+xs_cup+xrd_cup)+derivs.gxx*(xf2_cup+2*xf_xs_cup)/2 ... 
                + derivs.gxxx*(xf3_cup)/6; 
        elseif orderApp==4 
            xf2_cup    = kron(xf_cup,xf_cup); 
            xf3_cup    = kron(xf2_cup,xf_cup); 
            xf_xs_cup  = kron(xf_cup,xs_cup); 
            xf4_cup    = kron(xf3_cup,xf_cup); 
            xf2_xs_cup = kron(xf_cup,xf_xs_cup); 
            xs2_cup    = kron(xs_cup,xs_cup); 
            xf_xrd_cup = kron(xf_cup,xrd_cup); 
            yVec_cup   = model.g0 + derivs.gx*(xf_cup+xs_cup+xrd_cup+x4th_cup) ... 
                + derivs.gxx*(xf2_cup+2*xf_xs_cup+2*xf_xrd_cup+xs2_cup)/2 ... 
                + derivs.gxxx*(xf3_cup+3*xf2_xs_cup)/6 ... 
                + derivs.gxxxx*xf4_cup/24; 
        elseif orderApp==5 
            xf2_cup    = kron(xf_cup,xf_cup); 
            xf3_cup    = kron(xf2_cup,xf_cup); 
            xf_xs_cup  = kron(xf_cup,xs_cup); 
            xf4_cup    = kron(xf3_cup,xf_cup); 
            xf2_xs_cup = kron(xf_cup,xf_xs_cup); 
            xs2_cup    = kron(xs_cup,xs_cup); 
            xf_xrd_cup = kron(xf_cup,xrd_cup); 
            xf5_cup    = kron(xf4_cup,xf_cup); 
            xf3_xs_cup = kron(xf_cup,xf2_xs_cup); 
            xf_xs2_cup = kron(xf_cup,xs2_cup); 
            xf2_xrd_cup= kron(xf_cup,xf_xrd_cup); 
            xs_xrd_cup = kron(xs_cup,xrd_cup); 
            xf_x4th_cup= kron(xf_cup,x4th_cup); 
            yVec_cup = model.g0 + derivs.gx*(xf_cup+xs_cup+xrd_cup+x4th_cup+x5th_cup) ... 
                + derivs.gxx*(xf2_cup+2*xf_xs_cup+2*xf_xrd_cup+2*xf_x4th_cup+xs2_cup+2*xs_xrd_cup)/2 ... 
                + derivs.gxxx*(xf3_cup+3*xf2_xs_cup+3*xf2_xrd_cup+3*xf_xs2_cup)/6 ... 
                + derivs.gxxxx*(xf4_cup+4*xf3_xs_cup)/24 ... 
                + derivs.gxxxxx*xf5_cup/120; 
        end 
        c1_cup = yVec_cup(1,1);
        c2_cup = yVec_cup(2,1);
        c3_cup = yVec_cup(3,1);
        c4_cup = yVec_cup(4,1);
        c5_cup = yVec_cup(5,1);
        c6_cup = yVec_cup(6,1);
        c7_cup = yVec_cup(7,1);
        c8_cup = yVec_cup(8,1);
        c9_cup = yVec_cup(9,1);
        c10_cup = yVec_cup(10,1);
        iv1_cup = yVec_cup(11,1);
        iv2_cup = yVec_cup(12,1);
        iv3_cup = yVec_cup(13,1);
        iv4_cup = yVec_cup(14,1);
        iv5_cup = yVec_cup(15,1);
        iv6_cup = yVec_cup(16,1);
        iv7_cup = yVec_cup(17,1);
        iv8_cup = yVec_cup(18,1);
        iv9_cup = yVec_cup(19,1);
        iv10_cup = yVec_cup(20,1);
        la_cup = yVec_cup(21,1);
 
        % Getting the states at time t+1 
        k1_cup = model.h0(1,1) + xf_cup(1,1) + xs_cup(1,1) + xrd_cup(1,1) + x4th_cup(1,1) + x5th_cup(1,1);
        k2_cup = model.h0(2,1) + xf_cup(2,1) + xs_cup(2,1) + xrd_cup(2,1) + x4th_cup(2,1) + x5th_cup(2,1);
        k3_cup = model.h0(3,1) + xf_cup(3,1) + xs_cup(3,1) + xrd_cup(3,1) + x4th_cup(3,1) + x5th_cup(3,1);
        k4_cup = model.h0(4,1) + xf_cup(4,1) + xs_cup(4,1) + xrd_cup(4,1) + x4th_cup(4,1) + x5th_cup(4,1);
        k5_cup = model.h0(5,1) + xf_cup(5,1) + xs_cup(5,1) + xrd_cup(5,1) + x4th_cup(5,1) + x5th_cup(5,1);
        k6_cup = model.h0(6,1) + xf_cup(6,1) + xs_cup(6,1) + xrd_cup(6,1) + x4th_cup(6,1) + x5th_cup(6,1);
        k7_cup = model.h0(7,1) + xf_cup(7,1) + xs_cup(7,1) + xrd_cup(7,1) + x4th_cup(7,1) + x5th_cup(7,1);
        k8_cup = model.h0(8,1) + xf_cup(8,1) + xs_cup(8,1) + xrd_cup(8,1) + x4th_cup(8,1) + x5th_cup(8,1);
        k9_cup = model.h0(9,1) + xf_cup(9,1) + xs_cup(9,1) + xrd_cup(9,1) + x4th_cup(9,1) + x5th_cup(9,1);
        k10_cup = model.h0(10,1) + xf_cup(10,1) + xs_cup(10,1) + xrd_cup(10,1) + x4th_cup(10,1) + x5th_cup(10,1);
        a1_cup = model.h0(11,1) + xf_cup(11,1) + xs_cup(11,1) + xrd_cup(11,1) + x4th_cup(11,1) + x5th_cup(11,1);
        a2_cup = model.h0(12,1) + xf_cup(12,1) + xs_cup(12,1) + xrd_cup(12,1) + x4th_cup(12,1) + x5th_cup(12,1);
        a3_cup = model.h0(13,1) + xf_cup(13,1) + xs_cup(13,1) + xrd_cup(13,1) + x4th_cup(13,1) + x5th_cup(13,1);
        a4_cup = model.h0(14,1) + xf_cup(14,1) + xs_cup(14,1) + xrd_cup(14,1) + x4th_cup(14,1) + x5th_cup(14,1);
        a5_cup = model.h0(15,1) + xf_cup(15,1) + xs_cup(15,1) + xrd_cup(15,1) + x4th_cup(15,1) + x5th_cup(15,1);
        a6_cup = model.h0(16,1) + xf_cup(16,1) + xs_cup(16,1) + xrd_cup(16,1) + x4th_cup(16,1) + x5th_cup(16,1);
        a7_cup = model.h0(17,1) + xf_cup(17,1) + xs_cup(17,1) + xrd_cup(17,1) + x4th_cup(17,1) + x5th_cup(17,1);
        a8_cup = model.h0(18,1) + xf_cup(18,1) + xs_cup(18,1) + xrd_cup(18,1) + x4th_cup(18,1) + x5th_cup(18,1);
        a9_cup = model.h0(19,1) + xf_cup(19,1) + xs_cup(19,1) + xrd_cup(19,1) + x4th_cup(19,1) + x5th_cup(19,1);
        a10_cup = model.h0(20,1) + xf_cup(20,1) + xs_cup(20,1) + xrd_cup(20,1) + x4th_cup(20,1) + x5th_cup(20,1);
        %% Model Specific 
% START DISPLAYING f
f(1,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c1_cu)^(1/GAMA) - 1;
f(2,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c2_cu)^(1/GAMA) - 1;
f(3,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c3_cu)^(1/GAMA) - 1;
f(4,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c4_cu)^(1/GAMA) - 1;
f(5,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c5_cu)^(1/GAMA) - 1;
f(6,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c6_cu)^(1/GAMA) - 1;
f(7,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c7_cu)^(1/GAMA) - 1;
f(8,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c8_cu)^(1/GAMA) - 1;
f(9,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c9_cu)^(1/GAMA) - 1;
f(10,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c10_cu)^(1/GAMA) - 1;
f(11,1)=exp(-k1_cup)*(exp(iv1_cu) - exp(k1_cu)*(DELTA - 1)) - 1;
f(12,1)=exp(-k2_cup)*(exp(iv2_cu) - exp(k2_cu)*(DELTA - 1)) - 1;
f(13,1)=exp(-k3_cup)*(exp(iv3_cu) - exp(k3_cu)*(DELTA - 1)) - 1;
f(14,1)=exp(-k4_cup)*(exp(iv4_cu) - exp(k4_cu)*(DELTA - 1)) - 1;
f(15,1)=exp(-k5_cup)*(exp(iv5_cu) - exp(k5_cu)*(DELTA - 1)) - 1;
f(16,1)=exp(-k6_cup)*(exp(iv6_cu) - exp(k6_cu)*(DELTA - 1)) - 1;
f(17,1)=exp(-k7_cup)*(exp(iv7_cu) - exp(k7_cu)*(DELTA - 1)) - 1;
f(18,1)=exp(-k8_cup)*(exp(iv8_cu) - exp(k8_cu)*(DELTA - 1)) - 1;
f(19,1)=exp(-k9_cup)*(exp(iv9_cu) - exp(k9_cu)*(DELTA - 1)) - 1;
f(20,1)=exp(-k10_cup)*(exp(iv10_cu) - exp(k10_cu)*(DELTA - 1)) - 1;
f(21,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k1_cu)*exp(iv1_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k1_cup)*exp(iv1_cup))*((exp(-k1_cup)*exp(iv1_cup))/2 - DELTA/2 + 1) + (exp(a1_cup)*exp(k1_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k1_cu)*exp(iv1_cu)) - 1);
f(22,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k2_cu)*exp(iv2_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k2_cup)*exp(iv2_cup))*((exp(-k2_cup)*exp(iv2_cup))/2 - DELTA/2 + 1) + (exp(a2_cup)*exp(k2_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k2_cu)*exp(iv2_cu)) - 1);
f(23,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k3_cu)*exp(iv3_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k3_cup)*exp(iv3_cup))*((exp(-k3_cup)*exp(iv3_cup))/2 - DELTA/2 + 1) + (exp(a3_cup)*exp(k3_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k3_cu)*exp(iv3_cu)) - 1);
f(24,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k4_cu)*exp(iv4_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k4_cup)*exp(iv4_cup))*((exp(-k4_cup)*exp(iv4_cup))/2 - DELTA/2 + 1) + (exp(a4_cup)*exp(k4_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k4_cu)*exp(iv4_cu)) - 1);
f(25,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k5_cu)*exp(iv5_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k5_cup)*exp(iv5_cup))*((exp(-k5_cup)*exp(iv5_cup))/2 - DELTA/2 + 1) + (exp(a5_cup)*exp(k5_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k5_cu)*exp(iv5_cu)) - 1);
f(26,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k6_cu)*exp(iv6_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k6_cup)*exp(iv6_cup))*((exp(-k6_cup)*exp(iv6_cup))/2 - DELTA/2 + 1) + (exp(a6_cup)*exp(k6_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k6_cu)*exp(iv6_cu)) - 1);
f(27,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k7_cu)*exp(iv7_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k7_cup)*exp(iv7_cup))*((exp(-k7_cup)*exp(iv7_cup))/2 - DELTA/2 + 1) + (exp(a7_cup)*exp(k7_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k7_cu)*exp(iv7_cu)) - 1);
f(28,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k8_cu)*exp(iv8_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k8_cup)*exp(iv8_cup))*((exp(-k8_cup)*exp(iv8_cup))/2 - DELTA/2 + 1) + (exp(a8_cup)*exp(k8_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k8_cu)*exp(iv8_cu)) - 1);
f(29,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k9_cu)*exp(iv9_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k9_cup)*exp(iv9_cup))*((exp(-k9_cup)*exp(iv9_cup))/2 - DELTA/2 + 1) + (exp(a9_cup)*exp(k9_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k9_cu)*exp(iv9_cu)) - 1);
f(30,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k10_cu)*exp(iv10_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k10_cup)*exp(iv10_cup))*((exp(-k10_cup)*exp(iv10_cup))/2 - DELTA/2 + 1) + (exp(a10_cup)*exp(k10_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k10_cu)*exp(iv10_cu)) - 1);
f(31,1)=- ((PHI*exp(k10_cu)*(DELTA - exp(-k10_cu)*exp(iv10_cu))^2)/2 + (PHI*exp(k1_cu)*(DELTA - exp(-k1_cu)*exp(iv1_cu))^2)/2 + (PHI*exp(k2_cu)*(DELTA - exp(-k2_cu)*exp(iv2_cu))^2)/2 + (PHI*exp(k3_cu)*(DELTA - exp(-k3_cu)*exp(iv3_cu))^2)/2 + (PHI*exp(k4_cu)*(DELTA - exp(-k4_cu)*exp(iv4_cu))^2)/2 + (PHI*exp(k5_cu)*(DELTA - exp(-k5_cu)*exp(iv5_cu))^2)/2 + (PHI*exp(k6_cu)*(DELTA - exp(-k6_cu)*exp(iv6_cu))^2)/2 + (PHI*exp(k7_cu)*(DELTA - exp(-k7_cu)*exp(iv7_cu))^2)/2 + (PHI*exp(k8_cu)*(DELTA - exp(-k8_cu)*exp(iv8_cu))^2)/2 + (PHI*exp(k9_cu)*(DELTA - exp(-k9_cu)*exp(iv9_cu))^2)/2 + (exp(a10_cu)*exp(k10_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a1_cu)*exp(k1_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a2_cu)*exp(k2_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a3_cu)*exp(k3_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a4_cu)*exp(k4_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a5_cu)*exp(k5_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a6_cu)*exp(k6_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a7_cu)*exp(k7_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a8_cu)*exp(k8_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a9_cu)*exp(k9_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA))/(exp(c10_cu) + exp(c1_cu) + exp(c2_cu) + exp(c3_cu) + exp(c4_cu) + exp(c5_cu) + exp(c6_cu) + exp(c7_cu) + exp(c8_cu) + exp(c9_cu) + exp(iv1_cu) + exp(iv2_cu) + exp(iv3_cu) + exp(iv4_cu) + exp(iv5_cu) + exp(iv6_cu) + exp(iv7_cu) + exp(iv8_cu) + exp(iv9_cu) + exp(iv10_cu) - DELTA*exp(k10_cu) - DELTA*exp(k1_cu) - DELTA*exp(k2_cu) - DELTA*exp(k3_cu) - DELTA*exp(k4_cu) - DELTA*exp(k5_cu) - DELTA*exp(k6_cu) - DELTA*exp(k7_cu) - DELTA*exp(k8_cu) - DELTA*exp(k9_cu)) - 1;
f(32,1)=RHOA*log(exp(a1_cu)) - log(exp(a1_cup));
f(33,1)=RHOA*log(exp(a2_cu)) - log(exp(a2_cup));
f(34,1)=RHOA*log(exp(a3_cu)) - log(exp(a3_cup));
f(35,1)=RHOA*log(exp(a4_cu)) - log(exp(a4_cup));
f(36,1)=RHOA*log(exp(a5_cu)) - log(exp(a5_cup));
f(37,1)=RHOA*log(exp(a6_cu)) - log(exp(a6_cup));
f(38,1)=RHOA*log(exp(a7_cu)) - log(exp(a7_cup));
f(39,1)=RHOA*log(exp(a8_cu)) - log(exp(a8_cup));
f(40,1)=RHOA*log(exp(a9_cu)) - log(exp(a9_cup));
f(41,1)=RHOA*log(exp(a10_cu)) - log(exp(a10_cup));
% END DISPLAYING f
        Euler(:,1) = Euler(:,1) + wGrid(i,1)*f(:,1); 
    end 
    ResEuler(k,:) = transpose(Euler(:,1));
end 
