%Martin M. Andreasen, Jan 2019
warning off
addpath(genpath(pwd))
clear
close all
%clc
%% Settings
runPerturbation = 1;       % Chosen approximation method for simulation.
runPruning      = 1;
runExtendedPer  = 1; 

maxOrderApp     = 5;       %Order of approximation for the perturbation solution

numSim          = 1000;     %Number of periods simulated

% Configurations for the Extended Perturnation
setupEPer.Nmax            = 50;       %Max steps in the Extended Path
setupEPer.Nmin            = 50;        %Min steps in the Extended Path
setupEPer.maxDistSS       = 0.07*0;     %Max distance to SS when determining N
setupEPer.orderAppStart   = 1;         %Order of approximation for the starting values in the fixed point solver
setupEPer.fixedPointSolver= 1;         %1 for the Newton-Raphson solver,
                                       %2 Newton-Raphson solver with optimal delta,
                                       %3 for an LM algorithm that minimizes the residuals
                                       %4 for an LM algorithm that minimizes the weighted residuals
setupEPer.JacobianOption  = 3;         %1 for using numerical J and numerically solving the system J*x = -f
                                       %2 for using analytical J and numerically solving the system J*x = -f
                                       %3 for using analytical J and analytically solving the system J*x = -f
setupEPer.lambda0         = 1e-6;      %Tuining parameter for fixedPointSolver = 3
setupEPer.lambdaBackup    = 1e-2;      %Tuining parameter for the LM algorithm when used as backup
setupEPer.tolf            = 1D-6;      %Tolerance level for optimization problem
setupEPer.MaxIter         = 1D4;       %Maximum number of iterations allowed in the Extended Path algorithm
setupEPer.residualMax     = 1D-6*0;      %Max allowed value of a residual - for the hybrid simulator
setupEPer.MexOn           = 1*0;         %1 For using MEX-files, else 0
%% Model parameters
params.GAMA  = 1/4;
params.DELTA = 0.025;
params.BETTA = 0.99;
params.ALFA  = 0.36;
params.RHOA  = 0.99;
params.STDA  = 0.01*0;
params.PHI   = 0.5;
%% The perturbation approximation
load([pwd,'\ModelSpecification\setupModel'],'setupModel')
timePer = zeros(2,maxOrderApp);
for orderApp = 1:maxOrderApp
    if orderApp <= 3
        numRep = 10;
    else
        numRep = 1;
    end
    tic
    for k=1:numRep
        [model,errorMes] = perturbationDSGEmodel(params,orderApp,setupModel,setupEPer.MexOn);
    end
    timeSolvePer = toc/numRep;
    timePer(1,orderApp) = timeSolvePer;
    N = size(model.hx,1)/2;
    STDA = 0.01;
    for i=1:N
        model.eta(N+i,i) = STDA;
    end
    
    % Generating N.i.d. shocks to the economy
    rng(1,'twister');
    shocks = randn(model.ne,numSim); %Simulating from a random normal
    
    %% Perturbation: No pruning
    if runPerturbation == 1
        x0=model.h0;    % initial state is set at the steady state
        pruning=0;      % no pruning
        tic
        for k=1:numRep
            [ySim_Per,xSim_Per] = simPer(x0,model,shocks);
            %[ySim_Per,xSim_Per]=simul(x0,shocks,model.g0,model.h0,model.eta,model.derivs,orderApp,pruning);
        end
        timeSimPer = toc/numRep
        timePer(2,orderApp) = timeSimPer;
    end
end

%% Extended Perturbation
if runExtendedPer == 1
    tic
    setupEPer = infoToSetupEPer(model,setupEPer);
    timeSetupEPer = toc
    [ySim_EPer,xSim_EPer,diagnostic_EPer] = simExtendedPer(model,shocks,setupEPer);
    timeSimEPer = sum(diagnostic_EPer.timePerPeriod)+timeSetupEPer
    meanIter = mean(diagnostic_EPer.iter)
    resEP(1,:)  = log10([mean(abs(diagnostic_EPer.ferror(:)))...
                               sqrt(mean(diagnostic_EPer.ferror(:).^2)) ...
                               max(abs(diagnostic_EPer.ferror(:)))])

end
save(['SpeedTest_N',num2str(N)],'timePer','resEP','timeSimEPer','setupEPer')

