function [ResEuler,yGrid,xGrid] = EulerEqError(model,extendedPerOn,orderApp,setup)
 
% Some indices 
ny      = model.ny;        %Number of output variables
nx      = model.nx;        %Number of state variables 
 
%Unfold params
DELTA= model.params.DELTA;
BETTA= model.params.BETTA;
GAMA= model.params.GAMA;
PHI= model.params.PHI;
ALFA= model.params.ALFA;
RHOA= model.params.RHOA;
 
% The raw grid for the states 
if setup.xPointsDim == 0 
    xGrid = transpose(setup.xSim); 
else 
    dx       = (setup.upperx-setup.lowerx)/(setup.xPointsDim-1); 
    xGridtmp = zeros(nx,setup.xPointsDim); 
    for j=1:setup.xPointsDim 
        xGridtmp(:,j) = setup.lowerx(:,1)+(j-1)*dx(:,1); 
    end 
    tmp = cell(1,nx); 
    for i=1:nx 
        tmp(i) = mat2cell(xGridtmp(i,:),1,setup.xPointsDim); 
    end 
    xGrid = cartprod(tmp); 
end 
 
% The grid for numerical itegration 
tmp_e        = cell(1,model.ne); 
tmp_w        = cell(1,model.ne); 
for i=1:model.ne 
    tmp_e(i) = mat2cell(setup.GH_e,length(setup.GH_e),1); 
    tmp_w(i) = mat2cell(setup.GH_w,length(setup.GH_w),1); 
end 
eGrid = cartprod(tmp_e); 
wGrid = prod(cartprod(tmp_w),2); 
 
%% Building the residuals 
f        = zeros(ny+nx,1); 
ResEuler = zeros(size(xGrid,1),ny+nx); 
yGrid    = zeros(size(xGrid,1),ny); 
 
for k = 1:size(xGrid,1) 
    x_cu   = transpose(xGrid(k,:)); 
    % Getting the states at time t 
    k1_cu = model.h0(1,1) + x_cu(1,1);
    k2_cu = model.h0(2,1) + x_cu(2,1);
    k3_cu = model.h0(3,1) + x_cu(3,1);
    k4_cu = model.h0(4,1) + x_cu(4,1);
    a1_cu = model.h0(5,1) + x_cu(5,1);
    a2_cu = model.h0(6,1) + x_cu(6,1);
    a3_cu = model.h0(7,1) + x_cu(7,1);
    a4_cu = model.h0(8,1) + x_cu(8,1);
 
    % Getting the controls at time t and part of the states at time t+1 
    if extendedPerOn == 1
        [yVec_cu,xVec_cup,diagOut,Zopt] = policyFunctionEPer(x_cu+model.h0,model,setup.setupEPer);
        yVec_cup1 = Zopt(1:9,2)+model.g0;
    else
        [yVec_cu,xVec_cup]=policy(x_cu+model.h0,model.g0,model.h0,model.derivs,orderApp);
    end
    yGrid(k,:) = transpose(yVec_cu-model.g0);
    % The controls 
    c1_cu = yVec_cu(1,1);
    c2_cu = yVec_cu(2,1);
    c3_cu = yVec_cu(3,1);
    c4_cu = yVec_cu(4,1);
    iv1_cu = yVec_cu(5,1);
    iv2_cu = yVec_cu(6,1);
    iv3_cu = yVec_cu(7,1);
    iv4_cu = yVec_cu(8,1);
    la_cu = yVec_cu(9,1);
 
    Euler = zeros(ny+nx,1);
    for i=1:length(wGrid)
        x_cup = xVec_cup-model.h0; 
        % Adding shocks to the states
        x_cup = x_cup + model.eta*transpose(eGrid(i,:));
        % Getting controls at time t+1 
        if extendedPerOn == 1
            [yVec_cup,~,diag_cup] = policyFunctionEPer(x_cup+model.h0,model,setup.setupEPer);
        else
            yVec_cup = policy(x_cup+model.h0,model.g0,model.h0,model.derivs,orderApp);
        end
        %abs(yVec_cup-yVec_cup1)
        c1_cup = yVec_cup(1,1);
        c2_cup = yVec_cup(2,1);
        c3_cup = yVec_cup(3,1);
        c4_cup = yVec_cup(4,1);
        iv1_cup = yVec_cup(5,1);
        iv2_cup = yVec_cup(6,1);
        iv3_cup = yVec_cup(7,1);
        iv4_cup = yVec_cup(8,1);
        la_cup = yVec_cup(9,1);
 
        % Getting the states at time t+1 
        k1_cup = model.h0(1,1) + x_cup(1,1);
        k2_cup = model.h0(2,1) + x_cup(2,1);
        k3_cup = model.h0(3,1) + x_cup(3,1);
        k4_cup = model.h0(4,1) + x_cup(4,1);
        a1_cup = model.h0(5,1) + x_cup(5,1);
        a2_cup = model.h0(6,1) + x_cup(6,1);
        a3_cup = model.h0(7,1) + x_cup(7,1);
        a4_cup = model.h0(8,1) + x_cup(8,1);
        %% Model Specific 
% START DISPLAYING f
f(1,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c1_cu)^(1/GAMA) - 1;
f(2,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c2_cu)^(1/GAMA) - 1;
f(3,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c3_cu)^(1/GAMA) - 1;
f(4,1)=(exp(-la_cu)*(-(BETTA - 1)/(ALFA*BETTA))^(1/GAMA))/exp(c4_cu)^(1/GAMA) - 1;
f(5,1)=exp(-k1_cup)*(exp(iv1_cu) - exp(k1_cu)*(DELTA - 1)) - 1;
f(6,1)=exp(-k2_cup)*(exp(iv2_cu) - exp(k2_cu)*(DELTA - 1)) - 1;
f(7,1)=exp(-k3_cup)*(exp(iv3_cu) - exp(k3_cu)*(DELTA - 1)) - 1;
f(8,1)=exp(-k4_cup)*(exp(iv4_cu) - exp(k4_cu)*(DELTA - 1)) - 1;
f(9,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k1_cu)*exp(iv1_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k1_cup)*exp(iv1_cup))*((exp(-k1_cup)*exp(iv1_cup))/2 - DELTA/2 + 1) + (exp(a1_cup)*exp(k1_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k1_cu)*exp(iv1_cu)) - 1);
f(10,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k2_cu)*exp(iv2_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k2_cup)*exp(iv2_cup))*((exp(-k2_cup)*exp(iv2_cup))/2 - DELTA/2 + 1) + (exp(a2_cup)*exp(k2_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k2_cu)*exp(iv2_cu)) - 1);
f(11,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k3_cu)*exp(iv3_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k3_cup)*exp(iv3_cup))*((exp(-k3_cup)*exp(iv3_cup))/2 - DELTA/2 + 1) + (exp(a3_cup)*exp(k3_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k3_cu)*exp(iv3_cu)) - 1);
f(12,1)=-(exp(-la_cu)*(exp(la_cu)*(PHI*(DELTA - exp(-k4_cu)*exp(iv4_cu)) - 1) - BETTA*exp(la_cup)*(PHI*(DELTA - exp(-k4_cup)*exp(iv4_cup))*((exp(-k4_cup)*exp(iv4_cup))/2 - DELTA/2 + 1) + (exp(a4_cup)*exp(k4_cup)^(ALFA - 1)*(BETTA - 1))/BETTA - 1)))/(PHI*(DELTA - exp(-k4_cu)*exp(iv4_cu)) - 1);
f(13,1)=- ((PHI*exp(k1_cu)*(DELTA - exp(-k1_cu)*exp(iv1_cu))^2)/2 + (PHI*exp(k2_cu)*(DELTA - exp(-k2_cu)*exp(iv2_cu))^2)/2 + (PHI*exp(k3_cu)*(DELTA - exp(-k3_cu)*exp(iv3_cu))^2)/2 + (PHI*exp(k4_cu)*(DELTA - exp(-k4_cu)*exp(iv4_cu))^2)/2 + (exp(a1_cu)*exp(k1_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a2_cu)*exp(k2_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a3_cu)*exp(k3_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA) + (exp(a4_cu)*exp(k4_cu)^ALFA*(BETTA - 1))/(ALFA*BETTA))/(exp(c1_cu) + exp(c2_cu) + exp(c3_cu) + exp(c4_cu) + exp(iv1_cu) + exp(iv2_cu) + exp(iv3_cu) + exp(iv4_cu) - DELTA*exp(k1_cu) - DELTA*exp(k2_cu) - DELTA*exp(k3_cu) - DELTA*exp(k4_cu)) - 1;
f(14,1)=RHOA*log(exp(a1_cu)) - log(exp(a1_cup));
f(15,1)=RHOA*log(exp(a2_cu)) - log(exp(a2_cup));
f(16,1)=RHOA*log(exp(a3_cu)) - log(exp(a3_cup));
f(17,1)=RHOA*log(exp(a4_cu)) - log(exp(a4_cup));
% END DISPLAYING f
        Euler(:,1) = Euler(:,1) + wGrid(i,1)*f(:,1); 
    end 
    ResEuler(k,:) = transpose(Euler(:,1));
end 
