% This function checks the stability condition for the standard
% perturbation approximation using at most kMax iterations of the system forward in
% time. 
% This implementation diffes uses a recrusive preceedure 
% and stops if the condition is satisfied although k may be
% less than kMax
function out = stabilityTest(x,model,setup,extendedPerOn,orderApp,kMax,numCases)

dispOn = 0;
if any(isnan(x))
    out = struct();
    return
end
ne            = size(model.eta,2);
rng(1,'twister')
shocks        = randn(ne,kMax,numCases);

kValueStopTmp = zeros(1,numCases);
stableConTmp  = ones(1,numCases);
nx            = size(x,1);
epsValue      = 1D-6;

if extendedPerOn == 1
    yLevel0 = policyFunctionEPer(x+model.h0,model,setup.setupEPer);
else
    yLevel0 = policy(x+model.h0,model.g0,model.h0,model.derivs,orderApp);
end

%start of multiprocessing
setupEPer = setup.setupEPer;
numCPUs   = feature('numcores');
%parpool('local',numCPUs);
%parfor (j=1:numCases,numCPUs)
for j=1:numCases
    x_p = zeros(nx,nx);
    x_m = zeros(nx,nx);
    for i=1:nx
        % Positive step
        x_p(:,i) = x;
        x_p(i,i) = x_p(i,1)  +  epsValue;
        % Negative step
        x_m(:,i) = x;
        x_m(i,i) = x_m(i,1) -  epsValue;
    end
    k = 0;
    while stableConTmp(1,j) >= 1 && k<kMax        
        k = k + 1;
        for i=1:nx
            % For positive step
            if extendedPerOn == 1
                [~,xVec_cup] = policyFunctionEPer(x_p(:,i)+model.h0,model,setupEPer);
            else
                [~,xVec_cup]=policy(x_p(:,i)+model.h0,model.g0,model.h0,model.derivs,orderApp);
            end
            xVec_cup = xVec_cup + model.eta*squeeze(shocks(:,k,j));
            x_p(:,i) = xVec_cup-model.h0;
            
            % For negative step
            if extendedPerOn == 1
                [~,xVec_cup] = policyFunctionEPer(x_m(:,i)+model.h0,model,setupEPer);
            else
                [~,xVec_cup]=policy(x_m(:,i)+model.h0,model.g0,model.h0,model.derivs,orderApp);
            end
            xVec_cup = xVec_cup + model.eta*squeeze(shocks(:,k,j));
            x_m(:,i) = xVec_cup-model.h0;
        end
        % Numerical gradient
        tmp           = (x_p - x_m)/(2*epsValue);
        
        if any(any(isnan(tmp))) || any(any(isinf(tmp))) || any(any(abs(tmp)>1e100))
            stableConTmp(1,j) = inf;
            break
        else
            tmpValue          = sort(eig(tmp'*tmp),1,'descend');
            stableConTmp(1,j) = sqrt(tmpValue(1,1));
        end
        if dispOn == 1
            [k stableConTmp(1,j) model.Xss(1,1)*exp(x_p(1,1)) model.Xss(2,1)*exp(x_p(2,1))]
        end
    end
    
    kValueStopTmp(1,j) = k;
end
% End of multiprocessing
%delete(gcp('nocreate'))

stableCon  = max(stableConTmp);
kValueStop = max(kValueStopTmp);
% output
out.stableCon  = stableCon;
out.kValueStop = kValueStop;
out.yLevel     = yLevel0; 
end