%Martin M. Andreasen, Jan 2019
addpath(genpath(pwd))
clear
close all
%clc
%% Settings
runExtendedPer  = 1;        

GH_points       = 5*0;       %Number of points for numerical integration
xPointsDim      = 0;       %Number of points per state dimension for Accuracy grid. 
                           % If 0, then use sim path

maxOrderApp     = 5;       %Order of approximation for the perturbation solution
numSim          = 1000;     %Number of periods simulated

% Configurations for the Extended Perturnation
setupEPer.Nmax            = 50;       %Max steps in the Extended Path
setupEPer.Nmin            = 50;        %Min steps in the Extended Path
setupEPer.maxDistSS       = 0.07*0;     %Max distance to SS when determining N
setupEPer.orderAppStart   = 1;         %Order of approximation for the starting values in the fixed point solver
setupEPer.fixedPointSolver= 1;         %1 for the Newton-Raphson solver,
                                       %2 Newton-Raphson solver with optimal delta,
                                       %3 for an LM algorithm that minimizes the residuals
                                       %4 for an LM algorithm that minimizes the weighted residuals
setupEPer.JacobianOption  = 3;         %1 for using numerical J and numerically solving the system J*x = -f
                                       %2 for using analytical J and numerically solving the system J*x = -f
                                       %3 for using analytical J and analytically solving the system J*x = -f
setupEPer.lambda0         = 1e-6;      %Tuining parameter for fixedPointSolver = 3
setupEPer.lambdaBackup    = 1e-2;      %Tuining parameter for the LM algorithm when used as backup
setupEPer.tolf            = 1D-6;      %Tolerance level for optimization problem
setupEPer.MaxIter         = 1D4;       %Maximum number of iterations allowed in the Extended Path algorithm
setupEPer.residualMax     = 1D-5*0;      %Max allowed value of a residual - for the hybrid simulator
setupEPer.MexOn           = 1;         %1 For using MEX-files, else 0
%% Model parameters
params.GAMA  = 1/4;
params.DELTA = 0.025;
params.BETTA = 0.99;
params.ALFA  = 0.36;
params.RHOA  = 0.99;
params.STDA  = 0.01*0;
params.PHI   = 0.5;

%% The perturbation approximation
load([pwd,'\ModelSpecification\setupModel'],'setupModel')
[model,errorMes] = perturbationDSGEmodel(params,maxOrderApp,setupModel,setupEPer.MexOn);
N = size(model.hx,1)/2;
STDA = 0.01;
for i=1:N
    model.eta(N+i,i) = STDA;
end

% The setup structure for extended perturbation
setupEPer = infoToSetupEPer(model,setupEPer);

% Generating N.i.d. shocks to the economy
rng(1,'twister'); 
shocks = randn(model.ne,numSim); %Simulating from a random normal

%% Accuracy
% Standard deviations of the states in linearized model
nx   = size(model.eta,1);
varX = reshape((eye(nx^2)-kron(model.hx,model.hx))\reshape(model.eta*model.eta',nx^2,1),nx,nx);
stdX = sqrt(max(diag(varX),1D-6));

% Setup structure for accuracy
[setupEE.GH_e,setupEE.GH_w] = GaussHermite(GH_points);
setupEE.lowerx     = -3*stdX.*ones(length(model.nx),1);
setupEE.upperx     =  3*stdX.*ones(length(model.nx),1);
setupEE.xPointsDim = xPointsDim;
setupEE.setupEPer  = setupEPer;

% Simulating the states for accuracy
if xPointsDim == 0
    model.orderApp = maxOrderApp;
    [ySim_EPer,xSim_EPer,diagnostic_EPer] = simExtendedPer(model,shocks,setupEPer);
    setupEE.xSim   = xSim_EPer-repmat(model.h0,1,size(xSim_EPer,2));
    resEP(1,:)        = log10([mean(abs(diagnostic_EPer.ferror(:)))...
                               sqrt(mean(diagnostic_EPer.ferror(:).^2)) ...
                               max(abs(diagnostic_EPer.ferror(:)))])
end

resPer  = zeros(maxOrderApp,3);
for k=1:maxOrderApp
    model.orderApp = k;
    orderApp = k;
    % Standard Perturbation
    extendedPerOn      = 0;
    [ResEuler_Per,yGrid_1,xGrid] = EulerEqError(model,extendedPerOn,orderApp,setupEE);
    %[ResEuler_Per]            = EulerEqError_mp(model,extendedPerOn,orderApp,setupEE);
    MAE_Per            = mean(abs(ResEuler_Per(:)));
    RMSE_Per           = sqrt(mean(ResEuler_Per(:).^2));
    resPer(k,:)        = log10([MAE_Per RMSE_Per max(ResEuler_Per(:))])
    
end
fileName = ['Accuracy_N_',num2str(N),'_maxDistSS',num2str(setupEPer.maxDistSS),...
    '_residualMax',num2str(setupEPer.residualMax),'.mat'];
save(fileName,'resPer')