function out = getParamsTaylor(res,taylorRuleVer,rss)

RHOR = res.beta(2,1);
if taylorRuleVer <= 2
    KAPApai  = res.beta(3,1)/(1-RHOR);
    KAPAy    = res.beta(4,1)/(1-RHOR);
elseif taylorRuleVer == 3
    KAPApai  = res.beta(3,1)*rss/(1-RHOR);
    KAPAy    = res.beta(4,1)*rss/(1-RHOR);
end

% Standard errors for KAPApai and KAPAyhat by the delta method
% In general Q(KAPA,RHO) = KAPA/(1-RHO)
% dQ/dKAPA = 1/(1-RHO)
% dQ/RHO = KAPA*(1-RHO)^-2
% Thus, Q(KAPA,RHO) ~= Q(KAPA0,RHO0) + 1/(1-RHO0)*(KAPA-KAPA0) + KAPA0*(1-RHO0)^-2*(RHO-RHO0)
% Thus AVar(Q(KAPA,RHO)) = 1/(1-RHO0)^2*AVar(KAPA) + KAPA^2*(1-RHO)^-4*AVar(RHO)
%                          +2*KAPA/(1-RHO)^3*Cov(KAPA,RHO)
RHOR_se     = sqrt(res.V(2,2));
AVarRHOR    = res.V(2,2);
AVarKAPApai = res.V(3,3);
AVarKAPAy   = res.V(4,4);
CovPaiRho   = res.V(3,2);
CovYRho     = res.V(4,2);
if taylorRuleVer <= 2
    KAPApai_se  = sqrt(1/(1-RHOR)^2*AVarKAPApai + KAPApai^2*(1-RHOR)^-4*AVarRHOR + 2*KAPApai/(1-RHOR)^3*CovPaiRho);
    KAPAy_se    = sqrt(1/(1-RHOR)^2*AVarKAPAy   + KAPAy^2  *(1-RHOR)^-4*AVarRHOR + 2*KAPAy/(1-RHOR)^3*CovYRho);
elseif taylorRuleVer == 3
    KAPApai  = res.beta(3,1)/(1-RHOR);
    KAPAy    = res.beta(4,1)/(1-RHOR);
    KAPApai_se  = sqrt(rss^2/(1-RHOR)^2*AVarKAPApai + rss^2*KAPApai^2*(1-RHOR)^-4*AVarRHOR + 2*rss^2*KAPApai/(1-RHOR)^3*CovPaiRho);
    KAPAy_se    = sqrt(rss^2/(1-RHOR)^2*AVarKAPAy   + rss^2*KAPAy^2  *(1-RHOR)^-4*AVarRHOR + 2*rss^2*KAPAy/(1-RHOR)^3*CovYRho);
    % Adding the scaling by rss
    KAPApai  = res.beta(3,1)*rss/(1-RHOR);
    KAPAy    = res.beta(4,1)*rss/(1-RHOR);
end


out.RHOR        = RHOR;
out.KAPApai     = KAPApai;
out.KAPAy       = KAPAy;
out.RHOR_se     = RHOR_se;
out.KAPApai_se  = KAPApai_se;
out.KAPAy_se    = KAPAy_se;
