
% Loading data
tmp = xlsread('DataFRED_v3.xlsx');
data = tmp(:,2:end);
timeIndex  = x2mdate(tmp(:,1)); %datestr(x2mdate(tmp(:,1)));
dataLabels = {'$vMZM_t$','$R_t$','$\pi_t$','$\hat{y}_t$','$R_FED_t$'};
startNum = 198; %for starting in july 2008
[datestr(timeIndex(startNum+1)),' ', datestr(timeIndex(end))]

% Loading the Taylor rule estimates
olsR   = load('taylorRuleVer1_estimator1_startNum0.mat');
olsSRM = load('taylorRuleVer2_estimator1_startNum0.mat');
olsr   = load('taylorRuleVer3_estimator1_startNum0.mat');
%% The figure
close all
figureSize      = [0 0 1800 600+300]; % (0,0) size in x size in y
figure('Name','Taylor-rules','NumberTitle','off','Position',figureSize);
markerSizeValue = 9;
fontSizeValue   = 15;
fontSizeText    = 18;
LineWidthValue  = 2;

subplot(2,1,1)
hold on
plot(timeIndex(startNum+1:end),data(startNum+1:end,5),'-k')
plot(timeIndex(startNum+1:end),4*exp(olsr.rHat(startNum+1-1:end,1)),'-*k','markerSize',markerSizeValue)
plot(timeIndex(startNum+1:end),data(startNum+1:end,5)*0,':k')
axis([timeIndex(startNum+1) timeIndex(end) -0.005 0.03])
datetick('x','yyyy','keeplimits','keepticks')
timeIndexPlot   = 2010:2:2020;  
xticks(datenum(datetime(timeIndexPlot,1,1)))
xticklabels(num2cell(timeIndexPlot))
set(gca,'FontSize',fontSizeValue)
title('Effective Federal funds rate: ZLB period','interpreter','Latex','FontSize',fontSizeText)
legend({'Data','New Taylor-rule: $\log(r_t)$'},'Orientation','horizontal','Location','NorthEast','EdgeColor',[1 1 1],...
            'interpreter','Latex','FontSize',fontSizeText);

subplot(2,1,2)
hold on
plot(timeIndex(startNum+1:end),data(startNum+1:end,5),'-k')
plot(timeIndex(startNum+1:end),4*(exp(olsR.rHat(startNum+1-1:end,1))-1),'-ok','markerSize',markerSizeValue)
plot(timeIndex(startNum+1:end),4*(exp(olsSRM.rHat(startNum+1-1:end,1))-1),'-xk','markerSize',markerSizeValue+1)
plot(timeIndex(startNum+1:end),data(startNum+1:end,5)*0,':k')
axis([timeIndex(startNum+1) timeIndex(end) -0.005 0.03])
datetick('x','yyyy','keeplimits','keepticks')
timeIndexPlot   = 2010:2:2020;  
xticks(datenum(datetime(timeIndexPlot,1,1)))
xticklabels(num2cell(timeIndexPlot))
set(gca,'FontSize',fontSizeValue)
title('Effective Federal funds rate: ZLB period','interpreter','Latex','FontSize',fontSizeText)
legend({'Data','Standard Taylor-rule: $\log(R_t)$','Shadow Taylor-rule'},'Orientation','horizontal','Location','NorthEast','EdgeColor',[1 1 1],...
            'interpreter','Latex','FontSize',fontSizeText);
        
