clear

estimator     = 2;  %1 for OLS, 2 for IV
taylorRuleVer = 1;  %1 for standard, 2 for shadow rate model, 3 for exp(r)
rss           = 1/0.99-1;

% Loading data
tmp = xlsread('DataFRED_v3.xlsx');
data = tmp(:,2:end);
timeIndex  = x2mdate(tmp(:,1)); %datestr(x2mdate(tmp(:,1)));
dataLabels = {'$vMZM_t$','$R_t$','$\pi_t$','$\hat{y}_t$','$R_FED_t$'};
startNum   = 104*1*0;

[datestr(timeIndex(startNum+1)),' ', datestr(timeIndex(end))]
setup.r = data(startNum*1+1:end,5)/4;
setup.Infl = data(startNum*1+1:end,3)/4;
setup.y    = data(startNum*1+1:end,4);


%% OLS
if taylorRuleVer <= 2
    Y = log(1+setup.r(2:end,1));
elseif taylorRuleVer == 3
    Y = log(setup.r(2:end,1));
end
if estimator == 1
    if taylorRuleVer == 1 
        X = [ones(size(Y,1),1) log(1+setup.r(1:end-1,1))...
            setup.Infl(2:end,1) setup.y(2:end,1) ];
        res  = nwest(Y,X,4);
    elseif taylorRuleVer == 2
        params0 = ones(4,1);
        options = optimset('Display','iter','MaxIter',1D4,'MaxFunEvals',1D4,'TolFun',1D-4,'TolX',1D-4);
        res.beta = fminsearch(@ObjectBlackTaylorRule,params0,options,setup);
        [~,~,~,rHat] = ObjectBlackTaylorRule(res.beta,setup);
        X = [ones(size(Y,1),1) setup.r(1:end-1,1) ...
            setup.Infl(2:end,1) setup.y(2:end,1) ];
        res =nwest_BlackTaylor(Y,X,4,rHat,res.beta);
    elseif taylorRuleVer == 3
        X = [ones(size(Y,1),1) log(setup.r(1:end-1,1)) ...
            setup.Infl(2:end,1) setup.y(2:end,1) ];
        res  = nwest(Y,X,4);
    end
    R2   = res.rsqr
    paramsTaylor = getParamsTaylor(res,taylorRuleVer,rss);
    RHOr = res.beta(2,1)
    if taylorRuleVer <= 2
        KAPApai  = res.beta(3,1)/(1-RHOr)
        KAPAyhat = res.beta(4,1)/(1-RHOr)
	elseif taylorRuleVer == 3
        KAPApai  = res.beta(3,1)*rss/(1-RHOr)
        KAPAyhat = res.beta(4,1)*rss/(1-RHOr)
    end
    rHat     = res.yhat;
elseif estimator == 2
    %% IV estimation - 1 lags as instruments.
    if taylorRuleVer == 1
        Z = [ones(size(Y,1),1) log(1+setup.r(1:end-1,1)) setup.Infl(1:end-1,1) setup.y(1:end-1,1) ];
        X = [ones(size(Y,1),1) log(1+setup.r(1:end-1,1)) setup.Infl(2:end,1)   setup.y(2:end,1) ];
    elseif taylorRuleVer == 2
        error('IV for shadow rate model is not implemented')
    elseif taylorRuleVer == 3
        Z = [ones(size(Y,1),1) log(setup.r(1:end-1,1)) setup.Infl(1:end-1,1) setup.y(1:end-1,1) ];
        X = [ones(size(Y,1),1) log(setup.r(1:end-1,1)) setup.Infl(2:end,1) setup.y(2:end,1) ];
    end
    res  = nwest_iv(Y,X,Z,4);
    paramsTaylor = getParamsTaylor(res,taylorRuleVer,rss);
    % Check - using 2 stage SLS
    %resInfl = nwest(setup.Infl(2:end,1),Z,0);
    %inflHat = resInfl.yhat;
    %resyhat = nwest(setup.yhat(2:end,1),Z,0);
    %yHat = resyhat.yhat;
    %if taylorRuleVer == 1
    %    Xiv = [ones(size(Y,1),1) setup.r(1:end-1,1) inflHat yHat ];
    %elseif taylorRuleVer == 2
    %    Xiv = [ones(size(Y,1),1) log(setup.r(1:end-1,1)) inflHat yhatHat ];
    %end
    %resIVcheck     = nwest(Y,Xiv,4)
    res.rsqr
    RHOrIV    = res.beta(2,1)
    if taylorRuleVer == 1
        KAPApaiIV  = res.beta(3,1)/(1-RHOrIV)
        KAPAyhatIV = res.beta(4,1)/(1-RHOrIV) 
    elseif taylorRuleVer == 2
        error('IV for shadow rate model is not implemented')
    elseif taylorRuleVer == 3
        KAPApaiIV  = res.beta(3,1)*rss/(1-RHOrIV)
        KAPAyhatIV = res.beta(4,1)*rss/(1-RHOrIV)
    end
    rHat       = res.yhat;
end
fileName = ['taylorRuleVer',num2str(taylorRuleVer),'_estimator',num2str(estimator),...
        '_startNum',num2str(startNum),'.mat'];
save(fileName,'startNum','setup','timeIndex','res','rHat','Y','paramsTaylor')

close all
figure(1)
hold on
if taylorRuleVer <= 2
    plot(Y,'-k')
    plot(rHat,'-*r')
elseif taylorRuleVer == 3
    plot(exp(Y),'-k')
    plot(exp(rHat),'-*r')
end