function [ResEuler,yGrid,xGrid] = EulerEqError_PriceDisp(model,extendedPerOn,orderApp,setup)
 
% Some indices 
ny      = model.ny;        %Number of output variables
nx      = model.nx;        %Number of state variables 
 
%Unfold params
BETTA= model.params.BETTA;
PHIzero= model.params.PHIzero;
PHIone= model.params.PHIone;
PHItwo= model.params.PHItwo;
KAPAtwo= model.params.KAPAtwo;
DELTA= model.params.DELTA;
THETA= model.params.THETA;
ETA= model.params.ETA;
ALFA= model.params.ALFA;
RHOR= model.params.RHOR;
BETTAPAI= model.params.BETTAPAI;
BETTAY= model.params.BETTAY;
RHOA= model.params.RHOA;
IV= model.params.IV;
K= model.params.K;
R= model.params.R;
PAI= model.params.PAI;
Y= model.params.Y;
 
% The raw grid for the states 
if setup.xPointsDim == 0 
    xGrid = transpose(setup.xSim); 
else 
    dx       = (setup.upperx-setup.lowerx)/(setup.xPointsDim-1); 
    xGridtmp = zeros(nx,setup.xPointsDim); 
    for j=1:setup.xPointsDim 
        xGridtmp(:,j) = setup.lowerx(:,1)+(j-1)*dx(:,1); 
    end 
    tmp = cell(1,nx); 
    for i=1:nx 
        tmp(i) = mat2cell(xGridtmp(i,:),1,setup.xPointsDim); 
    end 
    xGrid = cartprod(tmp); 
end 
 
% The grid for numerical itegration 
tmp_e        = cell(1,model.ne); 
tmp_w        = cell(1,model.ne); 
for i=1:model.ne 
    tmp_e(i) = mat2cell(setup.GH_e,length(setup.GH_e),1); 
    tmp_w(i) = mat2cell(setup.GH_w,length(setup.GH_w),1); 
end 
eGrid = cartprod(tmp_e); 
wGrid = prod(cartprod(tmp_w),2); 
 
%% Building the residuals 
f        = zeros(ny+nx,1); 
ResEuler = zeros(size(xGrid,1),ny+nx); 
yGrid    = zeros(size(xGrid,1),ny); 
 
for k = 1:size(xGrid,1) 
    x_cu   = transpose(xGrid(k,:)); 
    % Getting the states at time t 
    k_cu = model.h0(1,1) + x_cu(1,1);
    s_cu = model.h0(2,1) + x_cu(2,1);
    r_ba1 = model.h0(3,1) + x_cu(3,1);
    a_cu = model.h0(4,1) + x_cu(4,1);
 
    % Getting the controls at time t and part of the states at time t+1 
    if extendedPerOn == 1
        [yVec_cu,xVec_cup,diagOut] = policyFunctionEPer(x_cu+model.h0,model,setup.setupEPer);
    else
        [yVec_cu,xVec_cup]=policy(x_cu+model.h0,model.g0,model.h0,model.derivs,orderApp);
    end
    yGrid(k,:) = transpose(yVec_cu-model.g0);
    % The controls 
    r_cu = yVec_cu(1,1);
    c_cu = yVec_cu(2,1);
    iv_cu = yVec_cu(3,1);
    pai_cu = yVec_cu(4,1);
    xtwo_cu = yVec_cu(5,1);
    y_cu = yVec_cu(6,1);
    w_cu = yVec_cu(7,1);
    mc_cu = yVec_cu(8,1);
    ptil_cu = yVec_cu(9,1);
    h_cu = yVec_cu(10,1);
    rk_cu = yVec_cu(11,1);
    q_cu = yVec_cu(12,1);
 
    % Using exact law of motion for the price dispersion index
    s_cu  = x_cu(2,1)+model.h0(2,1);
    %pai_cu = model.g0(4,1)+ model.gx(4,:)*x_cu; %using first order approx of infl
    
    %ptil_cuLevel = ((1 - ALFA*(1/exp(pai_cu))^(1-ETA))/(1-ALFA))^(1/(1-ETA));
    %ptil_cu = log(ptil_cuLevel);
    s_cup = (1-ALFA)^(1/(1-ETA))*(1-ALFA*exp(pai_cu)^(ETA-1))^(ETA/(ETA-1)) + ALFA*exp(pai_cu)^ETA*exp(s_cu);
    xVec_cup(2,1) = log(s_cup);
    
    Euler = zeros(ny+nx,1);
    for i=1:length(wGrid)
        x_cup = xVec_cup-model.h0; 
        % Adding shocks to the states
        x_cup = x_cup + model.eta*transpose(eGrid(i,:));
        % Getting controls at time t+1 
        if extendedPerOn == 1
            yVec_cup = policyFunctionEPer(x_cup+model.h0,model,setup.setupEPer);
        else
            yVec_cup = policy(x_cup+model.h0,model.g0,model.h0,model.derivs,orderApp);
        end
        r_cup = yVec_cup(1,1);
        c_cup = yVec_cup(2,1);
        iv_cup = yVec_cup(3,1);
        pai_cup = yVec_cup(4,1);
        xtwo_cup = yVec_cup(5,1);
        y_cup = yVec_cup(6,1);
        w_cup = yVec_cup(7,1);
        mc_cup = yVec_cup(8,1);
        ptil_cup = yVec_cup(9,1);
        h_cup = yVec_cup(10,1);
        rk_cup = yVec_cup(11,1);
        q_cup = yVec_cup(12,1);
 
        % Using exact relation for relative prices
        %pai_cup = model.g0(4,1)+ model.gx(4,:)*x_cup; %using first order approx of infl
        
        %ptil_cup = log(((1 - ALFA*(1/exp(pai_cup))^(1-ETA))/(1-ALFA))^(1/(1-ETA)));
        
        % Getting the states at time t+1 
        k_cup = model.h0(1,1) + x_cup(1,1);
        s_cup = model.h0(2,1) + x_cup(2,1);
        r_ba1p = model.h0(3,1) + x_cup(3,1);
        a_cup = model.h0(4,1) + x_cup(4,1);
        %% Model Specific 
% START DISPLAYING f
f(1,1)=- (BETTA*exp(-q_cu)*exp(c_cu)^PHItwo*(exp(q_cup)*(DELTA - 1) - exp(rk_cup) + (KAPAtwo*exp(q_cup)*(IV/K - exp(-k_cup)*exp(iv_cup))^2)/2 + KAPAtwo*exp(-k_cup)*exp(iv_cup)*exp(q_cup)*(IV/K - exp(-k_cup)*exp(iv_cup))))/exp(c_cup)^PHItwo - 1;
f(2,1)=(BETTA*exp(-pai_cup)*exp(r_cu)*exp(c_cu)^PHItwo)/exp(c_cup)^PHItwo - 1;
f(3,1)=(ETA*exp(-xtwo_cu)*((exp(mc_cu)*exp(y_cu))/exp(ptil_cu)^(ETA + 1) + (ALFA*BETTA*exp(xtwo_cup)*exp(c_cu)^PHItwo*(ETA - 1))/(ETA*exp(-pai_cup)^ETA*exp(c_cup)^PHItwo*(exp(-ptil_cup)*exp(ptil_cu))^(ETA + 1))))/(ETA - 1) - 1;
f(4,1)=(exp(-xtwo_cu)*exp(y_cu))/exp(ptil_cu)^ETA + (ALFA*BETTA*exp(-pai_cup)^(1 - ETA)*exp(-xtwo_cu)*exp(xtwo_cup)*exp(c_cu)^PHItwo)/(exp(c_cup)^PHItwo*(exp(-ptil_cup)*exp(ptil_cu))^ETA) - 1;
f(5,1)=exp(-y_cu)*exp(c_cu) + exp(-y_cu)*exp(iv_cu) - 1;
f(6,1)=exp(-s_cup)*exp(-y_cu)*exp(a_cu)*exp(h_cu)^(1 - THETA)*exp(k_cu)^THETA - 1;
f(7,1)=(PHIzero*exp(-w_cu)*exp(c_cu)^PHItwo)/(1 - exp(h_cu))^PHIone - 1;
f(8,1)=- (exp(-w_cu)*exp(a_cu)*exp(mc_cu)*(THETA - 1))/(exp(-k_cu)*exp(h_cu))^THETA - 1;
f(9,1)=RHOR*log(exp(r_ba1)/R) - log(exp(r_cu)/R) - (BETTAY*log(exp(y_cu)/Y) + BETTAPAI*log(exp(pai_cu)/PAI))*(RHOR - 1);
f(10,1)=ALFA*exp(-pai_cu)^(1 - ETA) - exp(ptil_cu)^(1 - ETA)*(ALFA - 1) - 1;
f(11,1)=THETA*exp(-rk_cu)*exp(a_cu)*exp(mc_cu)*(exp(-k_cu)*exp(h_cu))^(1 - THETA) - 1;
f(12,1)=exp(q_cu)*(KAPAtwo*(IV/K - exp(-k_cu)*exp(iv_cu)) + 1) - 1;
f(13,1)=exp(-k_cup)*exp(iv_cu) - exp(-k_cup)*exp(k_cu)*(DELTA - 1) - (KAPAtwo*exp(-k_cup)*exp(k_cu)*(IV/K - exp(-k_cu)*exp(iv_cu))^2)/2 - 1;
f(14,1)=ALFA*exp(-s_cup)*exp(s_cu)*exp(pai_cu)^ETA - (exp(-s_cup)*(ALFA - 1))/exp(ptil_cu)^ETA - 1;
f(15,1)=exp(-r_ba1p)*exp(r_cu) - 1;
f(16,1)=RHOA*log(exp(a_cu)) - log(exp(a_cup));
% END DISPLAYING f
        Euler(:,1) = Euler(:,1) + wGrid(i,1)*f(:,1); 
    end 
    ResEuler(k,:) = transpose(Euler(:,1));
end 
