%Martin M. Andreasen, Jan 2019
addpath(genpath(pwd))
clear
close all
%% Settings
stabilTestON        = 0;
computePolicyFuncON = 1;
plottingON          = 1;

GH_points           = 5;       % Number of points for numerical integration

orderAppMax         = 5;       %Maximal approximation order 

% Configurations for the Extended Perturnation
setupEPer.Nmax            = 300;       %Max steps in the Extended Path
setupEPer.Nmin            = 50;        %Min steps in the Extended Path
setupEPer.maxDistSS       = 0.01;      %Max distance to SS when determining N
setupEPer.orderAppStart   = orderAppMax;%Order of approximation for the starting values in the fixed point solver
setupEPer.fixedPointSolver= 1;         %1 for the Newton-Raphson solver,
                                       %2 Newton-Raphson solver with optimal delta,
                                       %3 for an LM algorithm that minimizes the residuals
                                       %4 for an LM algorithm that minimizes the weighted residuals
setupEPer.JacobianOption  = 3;         %1 for using numerical J and numerically solving the system J*x = -f
                                       %2 for using analytical J and numerically solving the system J*x = -f
                                       %3 for using analytical J and analytically solving the system J*x = -f
setupEPer.lambda0         = 1e-6;      %Tuining parameter for fixedPointSolver = 3
setupEPer.lambdaBackup    = 1e-2;      %Tuining parameter for the LM algorithm when used as backup
setupEPer.tolf            = 1e-10;     %Tolerance level for optimization problem
setupEPer.MaxIter         = 100;       %Maximum number of iterations allowed in the Extended Path algorithm
setupEPer.residualMax     = 0;         %Max allowed value of a residual - for the hybrid simulator
setupEPer.MexOn           = 1;         %1 For using MEX-files, else 0

%% Model parameters and The perturbation approximation
paramsVersion = 2; %jj;
[params,gridFactor_x] = getParamsVersion(paramsVersion);

gridFactor_x(1,1) = 4;  %Capital
gridFactor_x(2,1) = 4;  %Lagged short interest rate
gridFactor_x(4,1) = 3;  %technology

load('setupModel')
[model,errorMes] = perturbationDSGEmodel(params,orderAppMax,setupModel,setupEPer.MexOn);

% The setup structure for extended perturbation
setupEPer = infoToSetupEPer(model,setupEPer);

% Standard deviations of the states in linearized model
nx   = size(model.eta,1);
varX = reshape((eye(nx^2)-kron(model.hx,model.hx))\reshape(model.eta*model.eta',nx^2,1),nx,nx);
stdX = sqrt(max(diag(varX),1D-6));

% Information for constructing the grid of x_cu
[setupEE.GH_e,setupEE.GH_w] = GaussHermite(GH_points);
setupEE.lowerx     = -gridFactor_x.*stdX;
setupEE.upperx     =  gridFactor_x.*stdX;
setupEE.xPointsDim = 0;
setupEE.setupEPer  = setupEPer;
% Special limits for s_cu
if paramsVersion == 1 
    setupEE.lowerx(2,1) = 0.000;
    setupEE.upperx(2,1) = 0.08;
elseif paramsVersion == 2
    setupEE.lowerx(2,1) = 0.000;
    setupEE.upperx(2,1) = 0.2;
end

%% The stability test for perturbation
if stabilTestON  == 1
    BoundsStabilityTest = zeros(nx,2);
    for i=1:nx
        BoundsStabilityTest(i,:) = [setupEE.lowerx(i,1) setupEE.upperx(i,1)];
    end
    X             = cartprodMatrix(BoundsStabilityTest);
    numX          = size(X,1);
    kMax          = 1000;
    numCases      = 50;
    PerStabilTest = zeros(numX,3,orderAppMax);
    for m=1:orderAppMax
        m
        for i=1:numX
            x_cu = X(i,:)';
            extendedPerOn = 0;
            tmp = stabilityTest(x_cu,model,setupEE,extendedPerOn,m,kMax,numCases);
            PerStabilTest(i,:,m) = [tmp.stableCon, tmp.kValueStop tmp.yLevel(4,1)];
        end
    end
    inflMax = (1/params.ALFA)^(1/(params.ETA-1));    
    
    EPerStabilTest = zeros(numX,3,orderAppMax);
    % To ensure a smooth solution of the Extended Path
    setupEE.setupEPer.residualMax   = 0;
    setupEE.setupEPer.orderAppStart = 1;
    setupEPer.tolf                  = 1D-10;  
    for m=1:orderAppMax
        m
        for i=1:numX
            x_cu = X(i,:)';
            extendedPerOn = 1;
            tmp = stabilityTest(x_cu,model,setupEE,extendedPerOn,m,kMax,numCases);
            EPerStabilTest(i,:,m) = [tmp.stableCon, tmp.kValueStop tmp.yLevel(4,1)];
        end
    end
    
    fileName = ['StabilityTest_paramsVersion',num2str(paramsVersion),'.mat'];
    save(fileName,'PerStabilTest','EPerStabilTest','kMax','numCases','X');
end

%% Compute Data for Stability Plots
if computePolicyFuncON == 1
    GridPoints= 25;
    ny    = model.ny;

    Kcond     = setupEE.lowerx(1,1)   + setupEE.upperx(1,1)*0;
    Scond     = setupEE.lowerx(2,1)*0 + setupEE.upperx(2,1)*1;
    Rcond     = setupEE.lowerx(3,1)   + setupEE.upperx(3,1)*0;
    Acond     = setupEE.lowerx(3,1)   + setupEE.upperx(4,1)*0;

    % Price dispersion
    yPer_s      = zeros(ny,GridPoints,orderAppMax);
    xpPer_s     = zeros(nx,GridPoints,orderAppMax);
    yEPer_s     = zeros(ny,GridPoints,orderAppMax);
    xpEPer_s    = zeros(nx,GridPoints,orderAppMax);
    eulerPer_s  = zeros(GridPoints,orderAppMax);
    eulerEPer_s = zeros(GridPoints,orderAppMax);
    ds          = (setupEE.upperx(2,1)-setupEE.lowerx(2,1))/(GridPoints-1);
    sGrid       = setupEE.lowerx(2,1):ds:setupEE.upperx(2,1);
    x_cu        = zeros(nx,GridPoints);
    for m=1:orderAppMax
        for i1=1:GridPoints
            i1
            x_cu(:,i1) = [Kcond sGrid(1,i1) Rcond  Acond]';
            [yLevel,xpLevel]=policy(x_cu(:,i1)+model.h0,model.g0,model.h0,model.derivs,m);
            yPer_s(:,i1,m) = yLevel - model.g0;
            xpPer_s(:,i1,m) = xpLevel - model.h0;
            
            model.orderApp = m;
            [yLevel,xpLevel,diagOut] = policyFunctionEPer(x_cu(:,i1)+model.h0,model,setupEE.setupEPer);
            yEPer_s(:,i1,m) = yLevel - model.g0;
            xpEPer_s(:,i1,m) = xpLevel - model.h0;
        end
        % Accuracy
        setupEE.xSim     = x_cu;
        model.orderApp   = m;
        eulerPer_tmp     = EulerEqError(model,0,m,setupEE);
        eulerPer_s(:,m)  = mean(abs(eulerPer_tmp),2);
        eulerEPer_tmp    = EulerEqError(model,1,m,setupEE);
        eulerEPer_s(:,m) = mean(abs(eulerEPer_tmp),2);
    end
    plot([squeeze(yEPer_s(4,:,5))',squeeze(yPer_s(4,:,1))',squeeze(yPer_s(4,:,3))',squeeze(yPer_s(4,:,5))'])
    plot([squeeze(xpEPer_s(2,:,1))',squeeze(xpPer_s(2,:,1))',squeeze(xpPer_s(2,:,3))',squeeze(xpPer_s(2,:,5))'])
    plot(sGrid,[log10(squeeze(eulerEPer_s(:,5))),log10(squeeze(eulerPer_s(:,5)))])
    
    % Capital
    yPer_k      = zeros(ny,GridPoints,orderAppMax);
    xpPer_k     = zeros(nx,GridPoints,orderAppMax);
    yEPer_k     = zeros(ny,GridPoints,orderAppMax);
    xpEPer_k    = zeros(nx,GridPoints,orderAppMax);
    eulerPer_k  = zeros(GridPoints,orderAppMax);
    eulerEPer_k = zeros(GridPoints,orderAppMax);
    dk        = (setupEE.upperx(1,1)-setupEE.lowerx(1,1))/(GridPoints-1);
    kGrid     = setupEE.lowerx(1,1):dk:setupEE.upperx(1,1);
    for m=1:orderAppMax
        for i1=1:GridPoints
            i1
            x_cu = [kGrid(1,i1) Scond Rcond  Acond]';
            [yLevel,xpLevel]=policy(x_cu+model.h0,model.g0,model.h0,model.derivs,m);
            yPer_k(:,i1,m) = yLevel - model.g0;
            xpPer_k(:,i1,m) = xpLevel - model.h0;
            
            model.orderApp = m;
            [yLevel,xpLevel,diagOut] = policyFunctionEPer(x_cu+model.h0,model,setupEE.setupEPer);
            yEPer_k(:,i1,m) = yLevel - model.g0;
            xpEPer_k(:,i1,m) = xpLevel - model.h0;
        end
        % Accuracy
        setupEE.xSim     = x_cu;
        model.orderApp   = m;
        eulerPer_tmp     = EulerEqError(model,0,m,setupEE);
        eulerPer_k(:,m)  = mean(abs(eulerPer_tmp),2);
        eulerEPer_tmp    = EulerEqError(model,1,m,setupEE);
        eulerEPer_k(:,m) = mean(abs(eulerEPer_tmp),2);
    end
    plot(kGrid,[squeeze(yEPer_k(4,:,5))',squeeze(yPer_k(4,:,3))'])
    plot(kGrid,[squeeze(xpEPer_k(2,:,1))',squeeze(xpPer_k(2,:,2))'])
    plot(kGrid,[log10(squeeze(eulerEPer_k(:,5))),log10(squeeze(eulerPer_k(:,5)))])
    
    % Technology: Variation in a_cu 
    yPer_a      = zeros(ny,GridPoints,orderAppMax);
    xpPer_a     = zeros(nx,GridPoints,orderAppMax);
    yEPer_a     = zeros(ny,GridPoints,orderAppMax);
    xpEPer_a    = zeros(nx,GridPoints,orderAppMax);
    eulerPer_a  = zeros(GridPoints,orderAppMax);
    eulerEPer_a = zeros(GridPoints,orderAppMax);
    da          = (setupEE.upperx(4,1)-setupEE.lowerx(4,1))/(GridPoints-1);
    aGrid       = setupEE.lowerx(4,1):da:setupEE.upperx(4,1);
    for m=1:orderAppMax
        for i1=1:GridPoints
            i1
            x_cu = [Kcond Scond Rcond  aGrid(1,i1)]';
            [yLevel,xpLevel]=policy(x_cu+model.h0,model.g0,model.h0,model.derivs,m);
            yPer_a(:,i1,m) = yLevel - model.g0;
            xpPer_a(:,i1,m) = xpLevel - model.h0;
            
            model.orderApp = m;
            [yLevel,xpLevel,diagOut] = policyFunctionEPer(x_cu+model.h0,model,setupEE.setupEPer);
            yEPer_a(:,i1,m) = yLevel - model.g0;
            xpEPer_a(:,i1,m) = xpLevel - model.h0;
        end
        % Accuracy
        setupEE.xSim     = x_cu;
        model.orderApp   = m;
        eulerPer_tmp     = EulerEqError(model,0,m,setupEE);
        eulerPer_a(:,m)  = mean(abs(eulerPer_tmp),2);
        eulerEPer_tmp    = EulerEqError(model,1,m,setupEE);
        eulerEPer_a(:,m) = mean(abs(eulerEPer_tmp),2);
    end
    plot(aGrid,[squeeze(yEPer_a(4,:,1))',squeeze(yPer_a(4,:,3))'])
    plot(aGrid,[squeeze(xpEPer_a(2,:,1))',squeeze(xpPer_a(2,:,3))'])
    plot(aGrid,[log10(squeeze(eulerEPer_a(:,1))),log10(squeeze(eulerPer_a(:,3)))])
    fileName = ['StabilityData_paramsVersion',num2str(paramsVersion),'.mat'];
    save(fileName,'GridPoints','Kcond','Scond','Rcond','Acond',...
        'yPer_s','xpPer_s','yEPer_s','xpEPer_s','eulerPer_s','eulerEPer_s','sGrid',...
        'yPer_k','xpPer_k','yEPer_k','xpEPer_k','eulerPer_k','eulerEPer_k','kGrid',...
        'yPer_a','xpPer_a','yEPer_a','xpEPer_a','eulerPer_a','eulerEPer_a','aGrid');
end

%% Plotting
if plottingON == 1
    if computePolicyFuncON == 0
        fileName = ['StabilityData_paramsVersion',num2str(paramsVersion),'.mat'];
        load(fileName);
    end
    inflMax = (params.ALFA)^(1/(1-params.ETA))
    close all
    figureSize      = [0 0 1800 600+300]; % (0,0) size in x size in y
    figure('Name','Stability plot','NumberTitle','off','Position',figureSize);
    markerSize      = 5;
    fontSizeValue   = 12;
    fontSizeText    = 15;
    LineWidthValue  = 3;

    % Price dispersion
    subplot(3,3,1)
    hold on 
    plot(sGrid,yPer_s(4,:,1),'-b')
    plot(sGrid,yPer_s(4,:,3),'-ob')
    plot(sGrid,yPer_s(4,:,5),'-*b')
    plot(sGrid,yEPer_s(4,:,1),'-k','LineWidth',LineWidthValue)
    plot(sGrid,log(inflMax/params.PAI)*ones(1,length(sGrid)),'--k')
    set(gca,'FontSize',fontSizeValue)    
    xlabel('$\hat{s_t}$','interpreter','Latex','FontSize',fontSizeText)
    title('Inflation: $\hat{\pi_t}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight
    legend({'Per 1st','Per 3rd','Per 5th','EPer 5th','$\pi^{max}$'},'Orientation','horizontal','Location','NorthEast','EdgeColor',[1 1 1],...
            'interpreter','Latex','FontSize',fontSizeText);

    
    subplot(3,3,2)
    hold on 
    plot(sGrid,xpPer_s(2,:,1),'-b')
    plot(sGrid,xpPer_s(2,:,3),'-ob')
    plot(sGrid,xpPer_s(2,:,5),'-*b')
    plot(sGrid,xpEPer_s(2,:,5),'-k','LineWidth',LineWidthValue)
    set(gca,'FontSize',fontSizeValue)
    xlabel('$\hat{s_t}$','interpreter','Latex','FontSize',fontSizeText)
    title('Price dispersion: $\hat{s}_{t+1}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight
    
    subplot(3,3,3)
    hold on 
    plot(sGrid,log10(eulerPer_s(:,1)),'-b')
    plot(sGrid,log10(eulerPer_s(:,3)),'-ob')
    plot(sGrid,log10(eulerPer_s(:,5)),'-*b')
    plot(sGrid,log10(eulerEPer_s(:,5)),'-k','LineWidth',LineWidthValue)
    set(gca,'FontSize',fontSizeValue)
    title('Euler Errors: $\log_{10}$(MAE)','interpreter','Latex','FontSize',fontSizeText)
    xlabel('$\hat{s_t}$','interpreter','Latex','FontSize',fontSizeText)    
    axis tight
    
    % Capital
    subplot(3,3,4)
    hold on 
    plot(kGrid,yPer_k(4,:,1),'-b')
    plot(kGrid,yPer_k(4,:,3),'-ob')
    plot(kGrid,yPer_k(4,:,5),'-*b')
    plot(kGrid,yEPer_k(4,:,5),'-k','LineWidth',LineWidthValue)
    plot(kGrid,log(inflMax)*ones(1,length(kGrid)),'--k')
    set(gca,'FontSize',fontSizeValue)
    xlabel('$\hat{k_t}$','interpreter','Latex','FontSize',fontSizeText)
    title('Inflation: $\hat{\pi_t}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight
    
    subplot(3,3,5)
    hold on 
    plot(kGrid,xpPer_k(2,:,1),'-b')
    plot(kGrid,xpPer_k(2,:,3),'-ob')
    plot(kGrid,xpPer_k(2,:,5),'-*b')
    plot(kGrid,xpEPer_k(2,:,5),'-k','LineWidth',LineWidthValue)
    plot(0,0.05,'-r')
    set(gca,'FontSize',fontSizeValue)
    xlabel('$\hat{k_t}$','interpreter','Latex','FontSize',fontSizeText)
    title('Price dispersion: $\hat{s}_{t+1}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight
    
    subplot(3,3,6)
    hold on 
    plot(kGrid,log10(eulerPer_k(:,1)),'-b')
    plot(kGrid,log10(eulerPer_k(:,3)),'-ob')
    plot(kGrid,log10(eulerPer_k(:,5)),'-*b')
    plot(kGrid,log10(eulerEPer_k(:,5)),'-k','LineWidth',LineWidthValue)
    set(gca,'FontSize',fontSizeValue)
    title('Euler Errors: $\log_{10}$(MAE)','interpreter','Latex','FontSize',fontSizeText)
    xlabel('$\hat{k_t}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight    

	% Technology
    subplot(3,3,7)
    hold on 
    plot(aGrid,yPer_a(4,:,1),'-b')
    plot(aGrid,yPer_a(4,:,3),'-ob')
    plot(aGrid,yPer_a(4,:,5),'-*b')
    plot(aGrid,yEPer_a(4,:,5),'-k','LineWidth',LineWidthValue)
    plot(aGrid,log(inflMax)*ones(1,length(aGrid)),'--k')
    set(gca,'FontSize',fontSizeValue)    
    xlabel('$\hat{a_t}$','interpreter','Latex','FontSize',fontSizeText)
    title('Inflation: $\hat{\pi_t}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight
    
    subplot(3,3,8)
    hold on 
    plot(aGrid,xpPer_a(2,:,1),'-b')
    plot(aGrid,xpPer_a(2,:,3),'-ob')
    plot(aGrid,xpPer_a(2,:,5),'-*b')
    plot(aGrid,xpEPer_a(2,:,5),'-k','LineWidth',LineWidthValue)
    plot(0,0.01,'-r')
    set(gca,'FontSize',fontSizeValue)
    xlabel('$\hat{a_t}$','interpreter','Latex','FontSize',fontSizeText)
    title('Price dispersion: $\hat{s}_{t+1}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight
    
    subplot(3,3,9)
    hold on 
    plot(aGrid,log10(eulerPer_a(:,1)),'-b')
    plot(aGrid,log10(eulerPer_a(:,3)),'-ob')
    plot(aGrid,log10(eulerPer_a(:,5)),'-*b')
    plot(aGrid,log10(eulerEPer_a(:,5)),'-k','LineWidth',LineWidthValue)
    set(gca,'FontSize',fontSizeValue)
    title('Euler Errors: $\log_{10}$(MAE)','interpreter','Latex','FontSize',fontSizeText)
    xlabel('$\hat{a_t}$','interpreter','Latex','FontSize',fontSizeText)
    axis tight    
end
