%Martin M. Andreasen, Jan 2019
addpath(genpath(pwd))
clear
close all
%clc
%for jj=1:3
%for ii=0:1
%% Settings
numRep          = 1;
runExtendedPer  = 1;        

GH_points       = 5;       % Number of points for numerical integration
xPointsDim      = 0;       % Number of points per state dimension for Accuracy grid. 
                           % If 0, then use sim path
sameShocksON    = 0;       % 1 for conditioning on the same shocks when xPointsDim=0,
                           % 0 for conditioning on same states                    

numSim          = 1000;    %Number of periods simulated
burnIn          = 100;       %Burnin
orderAppMax     = 2;       %Maximal approximation order 

% Configurations for the Extended Perturnation
setupEPer.Nmax            = 200;    %Max steps in the Extended Path
setupEPer.Nmin            = 50;        %Min steps in the Extended Path
setupEPer.maxDistSS       = 0.01;     %Max distance to SS when determining N
setupEPer.orderAppStart   = 1;         %Order of approximation for the starting values in the fixed point solver
setupEPer.fixedPointSolver= 1;         %1 for the Newton-Raphson solver,
                                       %2 Newton-Raphson solver with optimal delta,
                                       %3 for an LM algorithm that minimizes the residuals
                                       %4 for an LM algorithm that minimizes the weighted residuals
setupEPer.JacobianOption  = 3;         %1 for using numerical J and numerically solving the system J*x = -f
                                       %2 for using analytical J and numerically solving the system J*x = -f
                                       %3 for using analytical J and analytically solving the system J*x = -f
setupEPer.lambda0         = 1e-6;      %Tuining parameter for fixedPointSolver = 3
setupEPer.lambdaBackup    = 1e-2;      %Tuining parameter for the LM algorithm when used as backup
setupEPer.tolf            = 1e-8;      %Tolerance level for optimization problem
setupEPer.MaxIter         = 100;       %Maximum number of iterations allowed in the Extended Path algorithm
setupEPer.residualMax     = 1D-4;      %Max allowed value of a residual - for the hybrid simulator
setupEPer.MexOn           = 1;         %1 For using MEX-files, else 0

%% Model parameters and The perturbation approximation
paramsVersion = 1;
[params,gridFactor_x] = getParamsVersion(paramsVersion);

load('setupModel')
[model,errorMes] = perturbationDSGEmodel(params,orderAppMax,setupModel,setupEPer.MexOn);

% The setup structure for extended perturbation
setupEPer = infoToSetupEPer(model,setupEPer);

% Standard deviations of the states in linearized model
nx   = size(model.eta,1);
varX = reshape((eye(nx^2)-kron(model.hx,model.hx))\reshape(model.eta*model.eta',nx^2,1),nx,nx);
stdX = sqrt(max(diag(varX),1D-6));

% Information for constructing the grid of x_cu
[setupEE.GH_e,setupEE.GH_w] = GaussHermite(GH_points);
setupEE.lowerx     = -gridFactor_x.*stdX;
setupEE.upperx     =  gridFactor_x.*stdX;
setupEE.xPointsDim = xPointsDim;
setupEE.setupEPer  = setupEPer;
% Special limits for s_cu
if paramsVersion == 1 
    setupEE.lowerx(2,1) = 0.000;
    setupEE.upperx(2,1) = 0.04;
elseif paramsVersion == 2
    setupEE.lowerx(2,1) = 0.000;
    setupEE.upperx(2,1) = 0.1;
end

% Simulating states from standard perturbation 
rng(10,'twister');
shocks = randn(model.ne,burnIn+numSim); %Simulating from a random normal
%save('shocks.mat')
%load('shocks.mat')
shocks             = shocks(model.ne,1:burnIn+numSim); %Simulating from a random normal

% Simulate the common states for evaluating accuacy.
if sameShocksON == 0 && xPointsDim == 0
    x0                 = model.h0;    % initial state is set at the steady state
    pruning            = 0;      % no pruning
    [ySim_Per,xSim_Per]= simul(x0,shocks,model.g0,model.h0,model.eta,model.derivs,orderAppMax,pruning);
    model.orderApp = 2;
    if runExtendedPer  == 1
        [ySim_EPer,xSim_EPer,diagnostic_EPer] = simExtendedPer(model,shocks,setupEPer);
        % The burnin
        xSim_EPer           = xSim_EPer(:,burnIn+1:end);
        setupEE.xSim       = xSim_EPer-repmat(model.h0,1,size(xSim_EPer,2));
    else
        % The burnin
        xSim_Per           = xSim_Per(:,burnIn+1:end);
        setupEE.xSim       = xSim_Per-repmat(model.h0,1,size(xSim_Per,2));
    end
end
%% Accuracy
resPer          = zeros(orderAppMax,4);
resEPer         = zeros(orderAppMax,4);
for k=1:orderAppMax
    orderApp                  = k;       %Order of approximation for the perturbation solution
    model.orderApp            = k;
    
    %% Standard Perturbation
    extendedPerOn      = 0;
    x0                 = model.h0;    % initial state is set at the steady state
    pruning            = 0;      % no pruning
    if sameShocksON == 1
        [ySim_Per,xSim_Per]= simul(x0,shocks,model.g0,model.h0,model.eta,model.derivs,k,pruning);
        %[ySim_EPer,xSim_EPer,diagnostic_EPer] = simExtendedPer(model,shocks,setupEPer);
        % The burnin
        xSim_Per           = xSim_Per(:,burnIn+1:end);
        setupEE.xSim       = xSim_Per-repmat(model.h0,1,size(xSim_Per,2));
    end
    tic
    for i=1:numRep
        %[ResEuler_Per,yGrid_Per,xGrid_Per] = EulerEqError_mp(model,extendedPerOn,orderApp,setupEE);
        [tmp,yGrid_Per,xGrid_Per] = EulerEqError(model,extendedPerOn,orderApp,setupEE);
    end
    timePer            = toc/numRep;
    ResEuler_Per(:,:,k)= tmp;
    totalResEuler_Per  = real(tmp(:))+imag(tmp(:));
    MAE_Per            = mean(abs(totalResEuler_Per));
    RMSE_Per           = sqrt(mean(totalResEuler_Per.^2));
    Max_Per            = max(abs(totalResEuler_Per));
    resPer(k,:)        = [MAE_Per RMSE_Per Max_Per timePer]
    
    %% Extended Perturbation
    if runExtendedPer == 1
        extendedPerOn = 1;
        % We evaluate accuracy on a simulated grid of states from the
        % Extended Path, meaning that the states will differ from those
        % used by standard perturbation       
        if xPointsDim  == 0 && sameShocksON == 1
            [ySim_EPer,xSim_EPer,diagnostic_EPer] = simExtendedPer(model,shocks,setupEPer);
            [y_cu,x_cup,diagOut] = policyFunctionEPer(xSim_Per(:,2),model,setupEPer);
            xSim_EPer           = xSim_EPer(:,burnIn+1:end);
            setupEE.xSim        = xSim_EPer-repmat(model.h0,1,size(xSim_EPer,2));
        end
        startTime                 = clock;
        %[tmp,yGrid_EPer]= EulerEqError(model,extendedPerOn,orderApp,setupEE);
        [tmp,yGrid_EPer]= EulerEqError_mp(model,extendedPerOn,orderApp,setupEE);
        endTime                   = clock;
        startTimeSec              = startTime(4)*60*60+startTime(5)*60+startTime(6);
        endTimeSec                = endTime(4)*60*60  +endTime(5)*60  +endTime(6);
        timeEper                  = endTimeSec-startTimeSec;
        ResEuler_EPer(:,:,k)      = tmp;
        totalResEuler_EPer         = real(ResEuler_EPer(:))+imag(ResEuler_EPer(:));
        MAE_EPer                   = mean(abs(totalResEuler_EPer));
        RMSE_EPer                  = sqrt(mean(totalResEuler_EPer.^2));
        Max_EPer                   = max(abs(totalResEuler_EPer));
        resEPer(k,:)               = [MAE_EPer RMSE_EPer Max_EPer timeEper]
    end
    
end
if xPointsDim > 0
    filename = ['Euler_paramsVersion',num2str(paramsVersion),'_grid_xPointsDim',num2str(xPointsDim),'_GH_points',num2str(GH_points),'_residualMax',num2str(setupEPer.residualMax),'_maxDistSS',num2str(setupEPer.maxDistSS),'.mat'];
else
    if runExtendedPer == 1
        filename = ['Euler_paramsVersion',num2str(paramsVersion),'_sim',num2str(numSim),'_sameShocksON',num2str(sameShocksON),'_GH_points',num2str(GH_points),'_residualMax',num2str(setupEPer.residualMax),'_maxDistSS',num2str(setupEPer.maxDistSS),'.mat'];
    else
        filename = ['Euler_paramsVersion',num2str(paramsVersion),'_sim',num2str(numSim),'_sameShocksON',num2str(sameShocksON),'_GH_points',num2str(GH_points),'_StandardPer.mat'];
    end
end
if runExtendedPer == 1
    save(filename,'resPer','resEPer','ResEuler_EPer','ResEuler_Per','model','setupEE',...
        'ySim_EPer','xSim_EPer','diagnostic_EPer','model','ySim_Per','xSim_Per')
elseif runExtendedPer == 0
    save(filename,'resPer','ResEuler_Per','model','setupEE','xGrid_Per','ySim_Per','xSim_Per')
end
