%Martin M. Andreasen, Jan 2019
addpath(genpath(pwd))
clear
close all
warning off
%clc
%% Settings
runPerturbation = 1;       % Chosen approximation method for simulation.
runExtendedPer  = 1; 

orderApp        = 2;       %Order of approximation for the perturbation solution

burnIn          = 100;
numSim          = 1000;     %Number of periods simulated

% Configurations for the Extended Perturnation
setupEPer.Nmax            = 200;       %Max steps in the Extended Path
setupEPer.Nmin            = 50;        %Min steps in the Extended Path
setupEPer.maxDistSS       = 0.03;     %Max distance to SS when determining N
setupEPer.orderAppStart   = 1;         %Order of approximation for the starting values in the fixed point solver
setupEPer.fixedPointSolver= 1;         %1 for the Newton-Raphson solver,
                                       %2 Newton-Raphson solver with optimal delta,
                                       %3 for an LM algorithm that minimizes the residuals
                                       %4 for an LM algorithm that minimizes the weighted residuals
setupEPer.JacobianOption  = 3;         %1 for using numerical J and numerically solving the system J*x = -f
                                       %2 for using analytical J and numerically solving the system J*x = -f
                                       %3 for using analytical J and analytically solving the system J*x = -f
setupEPer.lambda0         = 1e-6;      %Tuining parameter for fixedPointSolver = 3
setupEPer.lambdaBackup    = 1e-2;      %Tuining parameter for the LM algorithm when used as backup
setupEPer.tolf            = 1e-8;      %Tolerance level for optimization problem
setupEPer.MaxIter         = 1D4;       %Maximum number of iterations allowed in the Extended Path algorithm
setupEPer.residualMax     = 1D-4;    %Max allowed value of a residual - for the hybrid simulator
setupEPer.MexOn           = 1;         %1 For using MEX-files, else 0
%% Model parameters
paramsVersion = 1;
params = getParamsVersion(paramsVersion);

%% The perturbation approximation
load([pwd,'\ModelSpecification\setupModel'],'setupModel')
[model,errorMes] = perturbationDSGEmodel(params,orderApp,setupModel,setupEPer.MexOn);

% Generating N.i.d. shocks to the economy
rng(10,'twister'); 
shocks = randn(model.ne,burnIn+numSim); %Simulating from a random normal

%% Perturbation: No pruning
if runPerturbation == 1
    x0=model.h0;    % initial state is set at the steady state
    pruning=0;      % no pruning
    tic
    [ySim_Per,xSim_Per]=simul(x0,shocks,model.g0,model.h0,model.eta,model.derivs,orderApp,pruning); 
    timePer = toc
    [ySim_PerTrans,xSim_PerTrans] = untransformYandX(ySim_Per,model,xSim_Per);
end

%% Extended Perturbation
if runExtendedPer == 1
    setupEPer = infoToSetupEPer(model,setupEPer);
    [ySim_EPer,xSim_EPer,diagnostic_EPer] = simExtendedPer(model,shocks,setupEPer);
    timeEPer = sum(diagnostic_EPer.timePerPeriod)
    maxError = max(diagnostic_EPer.maxAbsfVal)
    %[y_cu,x_cup,diagOut] = policyFunctionEPer(xSim_EPer(:,995),model,setupEPer);
    [ySim_EPerTrans,xSim_EPerTrans] = untransformYandX(ySim_EPer,model,xSim_EPer);
    save(['SimSample_EPer_paramsVersion',num2str(paramsVersion),'.mat'],...
        'model','setupEPer','ySim_EPer','xSim_EPer','diagnostic_EPer');
else
    load(['SimSample_EPer_paramsVersion',num2str(paramsVersion),'.mat'],...
        'model','setupEPer','ySim_EPer','xSim_EPer','diagnostic_EPer');
end

